# -*- perl -*-
# DO NOT EDIT - This file is generated by UMMF; http://ummf.sourceforge.net 
# From template: $Id: PerlTemplate.txt,v 1.74 2004/08/01 03:50:07 kstephens Exp $

package UMMF::UML::MetaModel;

# use 5.6.1;
use strict;
use warnings;

#################################################################
# Version
#

our $VERSION = do { my @r = (q$Revision: 1.74 $ =~ /\d+/g); sprintf "%d." . "%03d" x $#r, @r };

=head1 NAME

UMMF::UML::MetaModel - Model package for Model UML 1.5 final/03-03-01;

=head1 SYNOPSIS

  use UMMF::UML::MetaModel;
  my $model = UMMF::UML::MetaModel->model;

=head1 DESCRIPTION

This package

=head1 USAGE

=head1 EXPORT

None exported.

=head1 AUTHOR

Kurt Stephens, kstephens@users.sourceforge.net 2003/04/15

=head1 CLASSIFIERS

The following is a list of all the Classifiers and their respective Perl modules defined in this Model.


=head2 C<Behavioral_Elements::Actions::Action_Foundation::Action>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action>.


=head2 C<Behavioral_Elements::Actions::Action_Foundation::ControlFlow>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::ControlFlow|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::ControlFlow>.


=head2 C<Behavioral_Elements::Actions::Action_Foundation::DataFlow>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow>.


=head2 C<Behavioral_Elements::Actions::Action_Foundation::InputPin>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::InputPin|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::InputPin>.


=head2 C<Behavioral_Elements::Actions::Action_Foundation::OutputPin>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin>.


=head2 C<Behavioral_Elements::Actions::Action_Foundation::Pin>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Pin|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Pin>.


=head2 C<Behavioral_Elements::Actions::Action_Foundation::PrimitiveAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::PrimitiveAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::PrimitiveAction>.


=head2 C<Behavioral_Elements::Actions::Action_Foundation::Procedure>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure>.


=head2 C<Behavioral_Elements::Actions::Action_Foundation::Variable>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Variable|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Variable>.


=head2 C<Behavioral_Elements::Actions::Collection_Actions::CollectionAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::CollectionAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::CollectionAction>.


=head2 C<Behavioral_Elements::Actions::Collection_Actions::Filter::FilterAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Filter::FilterAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Filter::FilterAction>.


=head2 C<Behavioral_Elements::Actions::Collection_Actions::Iterate::IterateAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Iterate::IterateAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Iterate::IterateAction>.


=head2 C<Behavioral_Elements::Actions::Collection_Actions::Map::MapAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Map::MapAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Map::MapAction>.


=head2 C<Behavioral_Elements::Actions::Collection_Actions::Reduce::ReduceAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Reduce::ReduceAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Reduce::ReduceAction>.


=head2 C<Behavioral_Elements::Actions::Composite_Actions::Clause>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::Clause>.


=head2 C<Behavioral_Elements::Actions::Composite_Actions::ConditionalAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::ConditionalAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::ConditionalAction>.


=head2 C<Behavioral_Elements::Actions::Composite_Actions::GroupAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::GroupAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::GroupAction>.


=head2 C<Behavioral_Elements::Actions::Composite_Actions::LoopAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction>.


=head2 C<Behavioral_Elements::Actions::Computation_Actions::ApplyFunctionAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::ApplyFunctionAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::ApplyFunctionAction>.


=head2 C<Behavioral_Elements::Actions::Computation_Actions::ArgumentSpecification>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::ArgumentSpecification|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::ArgumentSpecification>.


=head2 C<Behavioral_Elements::Actions::Computation_Actions::CodeAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::CodeAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::CodeAction>.


=head2 C<Behavioral_Elements::Actions::Computation_Actions::LiteralValueAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::LiteralValueAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::LiteralValueAction>.


=head2 C<Behavioral_Elements::Actions::Computation_Actions::MarshalAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::MarshalAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::MarshalAction>.


=head2 C<Behavioral_Elements::Actions::Computation_Actions::NullAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::NullAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::NullAction>.


=head2 C<Behavioral_Elements::Actions::Computation_Actions::PrimitiveFunction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::PrimitiveFunction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::PrimitiveFunction>.


=head2 C<Behavioral_Elements::Actions::Computation_Actions::TestIdentifyAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::TestIdentifyAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::TestIdentifyAction>.


=head2 C<Behavioral_Elements::Actions::Computation_Actions::UnmarshalAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::UnmarshalAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::UnmarshalAction>.


=head2 C<Behavioral_Elements::Actions::Jump_Actions::HandlerAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Jump_Actions::HandlerAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Jump_Actions::HandlerAction>.


=head2 C<Behavioral_Elements::Actions::Jump_Actions::JumpAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Jump_Actions::JumpAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Jump_Actions::JumpAction>.


=head2 C<Behavioral_Elements::Actions::Jump_Actions::JumpHandler>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Jump_Actions::JumpHandler|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Jump_Actions::JumpHandler>.


=head2 C<Behavioral_Elements::Actions::Messaging_Actions::AsynchronousInvocationAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::AsynchronousInvocationAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::AsynchronousInvocationAction>.


=head2 C<Behavioral_Elements::Actions::Messaging_Actions::BroadcastSignalAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::BroadcastSignalAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::BroadcastSignalAction>.


=head2 C<Behavioral_Elements::Actions::Messaging_Actions::CallOperationAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::CallOperationAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::CallOperationAction>.


=head2 C<Behavioral_Elements::Actions::Messaging_Actions::ExplicitInvocationAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::ExplicitInvocationAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::ExplicitInvocationAction>.


=head2 C<Behavioral_Elements::Actions::Messaging_Actions::InvocationAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::InvocationAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::InvocationAction>.


=head2 C<Behavioral_Elements::Actions::Messaging_Actions::SendSignalAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::SendSignalAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::SendSignalAction>.


=head2 C<Behavioral_Elements::Actions::Messaging_Actions::SynchronousInvocationAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::SynchronousInvocationAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::SynchronousInvocationAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ClearAssociationAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ClearAssociationAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ClearAssociationAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::CreateLinkAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::CreateLinkAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::CreateLinkAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::CreateLinkObjectAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::CreateLinkObjectAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::CreateLinkObjectAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::DestroyLinkAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::DestroyLinkAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::DestroyLinkAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkEndCreationData>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkEndCreationData|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkEndCreationData>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkEndData>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkEndData|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkEndData>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::QualifierValue>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::QualifierValue|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::QualifierValue>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkObjectEndAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkObjectEndAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkObjectEndAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkObjectQualifierAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkObjectQualifierAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkObjectQualifierAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::WriteLinkAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::WriteLinkAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::WriteLinkAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::AddAttributeValueAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::AddAttributeValueAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::AddAttributeValueAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::AttributeAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::AttributeAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::AttributeAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::ClearAttributeAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::ClearAttributeAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::ClearAttributeAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::ReadAttributeAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::ReadAttributeAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::ReadAttributeAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::RemoveAttributeValueAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::RemoveAttributeValueAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::RemoveAttributeValueAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::WriteAttributeAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::WriteAttributeAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::WriteAttributeAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::CreateObjectAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::CreateObjectAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::CreateObjectAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::DestroyObjectAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::DestroyObjectAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::DestroyObjectAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::ReadIsClassifiedObjectAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::ReadIsClassifiedObjectAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::ReadIsClassifiedObjectAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::ReclassifyObjectAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::ReclassifyObjectAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::ReclassifyObjectAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::CallProcedureAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::CallProcedureAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::CallProcedureAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::ReadExtentAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::ReadExtentAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::ReadExtentAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::ReadSelfAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::ReadSelfAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::ReadSelfAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::StartObjectStateMachineAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::StartObjectStateMachineAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::StartObjectStateMachineAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::AddVariableValueAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::AddVariableValueAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::AddVariableValueAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::ClearVariableAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::ClearVariableAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::ClearVariableAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::ReadVariableAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::ReadVariableAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::ReadVariableAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::RemoveVariableValueAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::RemoveVariableValueAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::RemoveVariableValueAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::VariableAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::VariableAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::VariableAction>.


=head2 C<Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::WriteVariableAction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::WriteVariableAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::WriteVariableAction>.


=head2 C<Behavioral_Elements::Activity_Graphs::ActionState>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::ActionState|UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::ActionState>.


=head2 C<Behavioral_Elements::Activity_Graphs::ActivityGraph>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::ActivityGraph|UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::ActivityGraph>.


=head2 C<Behavioral_Elements::Activity_Graphs::ClassifierInState>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::ClassifierInState|UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::ClassifierInState>.


=head2 C<Behavioral_Elements::Activity_Graphs::ObjectFlowState>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::ObjectFlowState|UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::ObjectFlowState>.


=head2 C<Behavioral_Elements::Activity_Graphs::Partition>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::Partition|UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::Partition>.


=head2 C<Behavioral_Elements::Activity_Graphs::SubactivityState>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::SubactivityState|UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::SubactivityState>.


=head2 C<Behavioral_Elements::Collaborations::AssociationEndRole>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::AssociationEndRole|UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::AssociationEndRole>.


=head2 C<Behavioral_Elements::Collaborations::AssociationRole>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::AssociationRole|UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::AssociationRole>.


=head2 C<Behavioral_Elements::Collaborations::ClassifierRole>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::ClassifierRole|UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::ClassifierRole>.


=head2 C<Behavioral_Elements::Collaborations::Collaboration>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::Collaboration|UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::Collaboration>.


=head2 C<Behavioral_Elements::Collaborations::CollaborationInstanceSet>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::CollaborationInstanceSet|UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::CollaborationInstanceSet>.


=head2 C<Behavioral_Elements::Collaborations::Interaction>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::Interaction|UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::Interaction>.


=head2 C<Behavioral_Elements::Collaborations::InteractionInstanceSet>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::InteractionInstanceSet|UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::InteractionInstanceSet>.


=head2 C<Behavioral_Elements::Collaborations::Message>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::Message|UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::Message>.


=head2 C<Behavioral_Elements::Common_Behavior::AttributeLink>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::AttributeLink|UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::AttributeLink>.


=head2 C<Behavioral_Elements::Common_Behavior::ComponentInstance>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::ComponentInstance|UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::ComponentInstance>.


=head2 C<Behavioral_Elements::Common_Behavior::DataValue>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::DataValue|UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::DataValue>.


=head2 C<Behavioral_Elements::Common_Behavior::Exception>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Exception|UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Exception>.


=head2 C<Behavioral_Elements::Common_Behavior::Instance>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Instance|UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Instance>.


=head2 C<Behavioral_Elements::Common_Behavior::Link>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Link|UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Link>.


=head2 C<Behavioral_Elements::Common_Behavior::LinkEnd>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::LinkEnd|UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::LinkEnd>.


=head2 C<Behavioral_Elements::Common_Behavior::LinkObject>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::LinkObject|UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::LinkObject>.


=head2 C<Behavioral_Elements::Common_Behavior::NodeInstance>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::NodeInstance|UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::NodeInstance>.


=head2 C<Behavioral_Elements::Common_Behavior::Object>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Object|UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Object>.


=head2 C<Behavioral_Elements::Common_Behavior::Procedure>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Procedure|UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Procedure>.


=head2 C<Behavioral_Elements::Common_Behavior::Reception>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Reception|UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Reception>.


=head2 C<Behavioral_Elements::Common_Behavior::Signal>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Signal|UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Signal>.


=head2 C<Behavioral_Elements::Common_Behavior::Stimulus>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Stimulus|UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Stimulus>.


=head2 C<Behavioral_Elements::Common_Behavior::SubsystemInstance>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::SubsystemInstance|UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::SubsystemInstance>.


=head2 C<Behavioral_Elements::State_Machines::CallEvent>

See L<UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::CallEvent|UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::CallEvent>.


=head2 C<Behavioral_Elements::State_Machines::ChangeEvent>

See L<UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::ChangeEvent|UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::ChangeEvent>.


=head2 C<Behavioral_Elements::State_Machines::CompositeState>

See L<UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::CompositeState|UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::CompositeState>.


=head2 C<Behavioral_Elements::State_Machines::Event>

See L<UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::Event|UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::Event>.


=head2 C<Behavioral_Elements::State_Machines::FinalState>

See L<UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::FinalState|UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::FinalState>.


=head2 C<Behavioral_Elements::State_Machines::Guard>

See L<UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::Guard|UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::Guard>.


=head2 C<Behavioral_Elements::State_Machines::Pseudostate>

See L<UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::Pseudostate|UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::Pseudostate>.


=head2 C<Behavioral_Elements::State_Machines::SignalEvent>

See L<UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::SignalEvent|UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::SignalEvent>.


=head2 C<Behavioral_Elements::State_Machines::SimpleState>

See L<UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::SimpleState|UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::SimpleState>.


=head2 C<Behavioral_Elements::State_Machines::State>

See L<UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::State|UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::State>.


=head2 C<Behavioral_Elements::State_Machines::StateMachine>

See L<UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::StateMachine|UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::StateMachine>.


=head2 C<Behavioral_Elements::State_Machines::StateVertex>

See L<UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::StateVertex|UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::StateVertex>.


=head2 C<Behavioral_Elements::State_Machines::StubState>

See L<UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::StubState|UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::StubState>.


=head2 C<Behavioral_Elements::State_Machines::SubmachineState>

See L<UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::SubmachineState|UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::SubmachineState>.


=head2 C<Behavioral_Elements::State_Machines::SynchState>

See L<UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::SynchState|UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::SynchState>.


=head2 C<Behavioral_Elements::State_Machines::TimeEvent>

See L<UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::TimeEvent|UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::TimeEvent>.


=head2 C<Behavioral_Elements::State_Machines::Transition>

See L<UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::Transition|UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::Transition>.


=head2 C<Behavioral_Elements::Use_Cases::Actor>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::Actor|UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::Actor>.


=head2 C<Behavioral_Elements::Use_Cases::Extend>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::Extend|UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::Extend>.


=head2 C<Behavioral_Elements::Use_Cases::ExtensionPoint>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::ExtensionPoint|UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::ExtensionPoint>.


=head2 C<Behavioral_Elements::Use_Cases::Include>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::Include|UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::Include>.


=head2 C<Behavioral_Elements::Use_Cases::UseCase>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::UseCase|UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::UseCase>.


=head2 C<Behavioral_Elements::Use_Cases::UseCaseInstance>

See L<UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::UseCaseInstance|UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::UseCaseInstance>.


=head2 C<Foundation::Core::Abstraction>

See L<UMMF::UML::MetaModel::Foundation::Core::Abstraction|UMMF::UML::MetaModel::Foundation::Core::Abstraction>.


=head2 C<Foundation::Core::Artifact>

See L<UMMF::UML::MetaModel::Foundation::Core::Artifact|UMMF::UML::MetaModel::Foundation::Core::Artifact>.


=head2 C<Foundation::Core::Association>

See L<UMMF::UML::MetaModel::Foundation::Core::Association|UMMF::UML::MetaModel::Foundation::Core::Association>.


=head2 C<Foundation::Core::AssociationClass>

See L<UMMF::UML::MetaModel::Foundation::Core::AssociationClass|UMMF::UML::MetaModel::Foundation::Core::AssociationClass>.


=head2 C<Foundation::Core::AssociationEnd>

See L<UMMF::UML::MetaModel::Foundation::Core::AssociationEnd|UMMF::UML::MetaModel::Foundation::Core::AssociationEnd>.


=head2 C<Foundation::Core::Attribute>

See L<UMMF::UML::MetaModel::Foundation::Core::Attribute|UMMF::UML::MetaModel::Foundation::Core::Attribute>.


=head2 C<Foundation::Core::BehavioralFeature>

See L<UMMF::UML::MetaModel::Foundation::Core::BehavioralFeature|UMMF::UML::MetaModel::Foundation::Core::BehavioralFeature>.


=head2 C<Foundation::Core::Binding>

See L<UMMF::UML::MetaModel::Foundation::Core::Binding|UMMF::UML::MetaModel::Foundation::Core::Binding>.


=head2 C<Foundation::Core::Class>

See L<UMMF::UML::MetaModel::Foundation::Core::Class|UMMF::UML::MetaModel::Foundation::Core::Class>.


=head2 C<Foundation::Core::Classifier>

See L<UMMF::UML::MetaModel::Foundation::Core::Classifier|UMMF::UML::MetaModel::Foundation::Core::Classifier>.


=head2 C<Foundation::Core::Comment>

See L<UMMF::UML::MetaModel::Foundation::Core::Comment|UMMF::UML::MetaModel::Foundation::Core::Comment>.


=head2 C<Foundation::Core::Component>

See L<UMMF::UML::MetaModel::Foundation::Core::Component|UMMF::UML::MetaModel::Foundation::Core::Component>.


=head2 C<Foundation::Core::Constraint>

See L<UMMF::UML::MetaModel::Foundation::Core::Constraint|UMMF::UML::MetaModel::Foundation::Core::Constraint>.


=head2 C<Foundation::Core::DataType>

See L<UMMF::UML::MetaModel::Foundation::Core::DataType|UMMF::UML::MetaModel::Foundation::Core::DataType>.


=head2 C<Foundation::Core::Dependency>

See L<UMMF::UML::MetaModel::Foundation::Core::Dependency|UMMF::UML::MetaModel::Foundation::Core::Dependency>.


=head2 C<Foundation::Core::Element>

See L<UMMF::UML::MetaModel::Foundation::Core::Element|UMMF::UML::MetaModel::Foundation::Core::Element>.


=head2 C<Foundation::Core::ElementOwnership>

See L<UMMF::UML::MetaModel::Foundation::Core::ElementOwnership|UMMF::UML::MetaModel::Foundation::Core::ElementOwnership>.


=head2 C<Foundation::Core::Enumeration>

See L<UMMF::UML::MetaModel::Foundation::Core::Enumeration|UMMF::UML::MetaModel::Foundation::Core::Enumeration>.


=head2 C<Foundation::Core::EnumerationLiteral>

See L<UMMF::UML::MetaModel::Foundation::Core::EnumerationLiteral|UMMF::UML::MetaModel::Foundation::Core::EnumerationLiteral>.


=head2 C<Foundation::Core::Feature>

See L<UMMF::UML::MetaModel::Foundation::Core::Feature|UMMF::UML::MetaModel::Foundation::Core::Feature>.


=head2 C<Foundation::Core::Flow>

See L<UMMF::UML::MetaModel::Foundation::Core::Flow|UMMF::UML::MetaModel::Foundation::Core::Flow>.


=head2 C<Foundation::Core::GeneralizableElement>

See L<UMMF::UML::MetaModel::Foundation::Core::GeneralizableElement|UMMF::UML::MetaModel::Foundation::Core::GeneralizableElement>.


=head2 C<Foundation::Core::Generalization>

See L<UMMF::UML::MetaModel::Foundation::Core::Generalization|UMMF::UML::MetaModel::Foundation::Core::Generalization>.


=head2 C<Foundation::Core::Interface>

See L<UMMF::UML::MetaModel::Foundation::Core::Interface|UMMF::UML::MetaModel::Foundation::Core::Interface>.


=head2 C<Foundation::Core::Method>

See L<UMMF::UML::MetaModel::Foundation::Core::Method|UMMF::UML::MetaModel::Foundation::Core::Method>.


=head2 C<Foundation::Core::ModelElement>

See L<UMMF::UML::MetaModel::Foundation::Core::ModelElement|UMMF::UML::MetaModel::Foundation::Core::ModelElement>.


=head2 C<Foundation::Core::Namespace>

See L<UMMF::UML::MetaModel::Foundation::Core::Namespace|UMMF::UML::MetaModel::Foundation::Core::Namespace>.


=head2 C<Foundation::Core::Node>

See L<UMMF::UML::MetaModel::Foundation::Core::Node|UMMF::UML::MetaModel::Foundation::Core::Node>.


=head2 C<Foundation::Core::Operation>

See L<UMMF::UML::MetaModel::Foundation::Core::Operation|UMMF::UML::MetaModel::Foundation::Core::Operation>.


=head2 C<Foundation::Core::Parameter>

See L<UMMF::UML::MetaModel::Foundation::Core::Parameter|UMMF::UML::MetaModel::Foundation::Core::Parameter>.


=head2 C<Foundation::Core::Permission>

See L<UMMF::UML::MetaModel::Foundation::Core::Permission|UMMF::UML::MetaModel::Foundation::Core::Permission>.


=head2 C<Foundation::Core::PresentationElement>

See L<UMMF::UML::MetaModel::Foundation::Core::PresentationElement|UMMF::UML::MetaModel::Foundation::Core::PresentationElement>.


=head2 C<Foundation::Core::Primitive>

See L<UMMF::UML::MetaModel::Foundation::Core::Primitive|UMMF::UML::MetaModel::Foundation::Core::Primitive>.


=head2 C<Foundation::Core::ProgrammingLanguageDataType>

See L<UMMF::UML::MetaModel::Foundation::Core::ProgrammingLanguageDataType|UMMF::UML::MetaModel::Foundation::Core::ProgrammingLanguageDataType>.


=head2 C<Foundation::Core::Relationship>

See L<UMMF::UML::MetaModel::Foundation::Core::Relationship|UMMF::UML::MetaModel::Foundation::Core::Relationship>.


=head2 C<Foundation::Core::StructuralFeature>

See L<UMMF::UML::MetaModel::Foundation::Core::StructuralFeature|UMMF::UML::MetaModel::Foundation::Core::StructuralFeature>.


=head2 C<Foundation::Core::TemplateArgument>

See L<UMMF::UML::MetaModel::Foundation::Core::TemplateArgument|UMMF::UML::MetaModel::Foundation::Core::TemplateArgument>.


=head2 C<Foundation::Core::TemplateParameter>

See L<UMMF::UML::MetaModel::Foundation::Core::TemplateParameter|UMMF::UML::MetaModel::Foundation::Core::TemplateParameter>.


=head2 C<Foundation::Core::Usage>

See L<UMMF::UML::MetaModel::Foundation::Core::Usage|UMMF::UML::MetaModel::Foundation::Core::Usage>.


=head2 C<Foundation::Data_Types::AggregationKind>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::AggregationKind|UMMF::UML::MetaModel::Foundation::Data_Types::AggregationKind>.


=head2 C<Foundation::Data_Types::ArgListExpression>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::ArgListExpression|UMMF::UML::MetaModel::Foundation::Data_Types::ArgListExpression>.


=head2 C<Foundation::Data_Types::Boolean>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::Boolean|UMMF::UML::MetaModel::Foundation::Data_Types::Boolean>.


=head2 C<Foundation::Data_Types::BooleanExpression>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::BooleanExpression|UMMF::UML::MetaModel::Foundation::Data_Types::BooleanExpression>.


=head2 C<Foundation::Data_Types::CallConcurrencyKind>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::CallConcurrencyKind|UMMF::UML::MetaModel::Foundation::Data_Types::CallConcurrencyKind>.


=head2 C<Foundation::Data_Types::ChangeableKind>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::ChangeableKind|UMMF::UML::MetaModel::Foundation::Data_Types::ChangeableKind>.


=head2 C<Foundation::Data_Types::Expression>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::Expression|UMMF::UML::MetaModel::Foundation::Data_Types::Expression>.


=head2 C<Foundation::Data_Types::Geometry>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::Geometry|UMMF::UML::MetaModel::Foundation::Data_Types::Geometry>.


=head2 C<Foundation::Data_Types::Integer>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::Integer|UMMF::UML::MetaModel::Foundation::Data_Types::Integer>.


=head2 C<Foundation::Data_Types::LocationReference>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::LocationReference|UMMF::UML::MetaModel::Foundation::Data_Types::LocationReference>.


=head2 C<Foundation::Data_Types::Mapping>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::Mapping|UMMF::UML::MetaModel::Foundation::Data_Types::Mapping>.


=head2 C<Foundation::Data_Types::MappingExpression>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::MappingExpression|UMMF::UML::MetaModel::Foundation::Data_Types::MappingExpression>.


=head2 C<Foundation::Data_Types::Multiplicity>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::Multiplicity|UMMF::UML::MetaModel::Foundation::Data_Types::Multiplicity>.


=head2 C<Foundation::Data_Types::MultiplicityRange>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::MultiplicityRange|UMMF::UML::MetaModel::Foundation::Data_Types::MultiplicityRange>.


=head2 C<Foundation::Data_Types::Name>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::Name|UMMF::UML::MetaModel::Foundation::Data_Types::Name>.


=head2 C<Foundation::Data_Types::OrderingKind>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::OrderingKind|UMMF::UML::MetaModel::Foundation::Data_Types::OrderingKind>.


=head2 C<Foundation::Data_Types::ParameterDirectionKind>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::ParameterDirectionKind|UMMF::UML::MetaModel::Foundation::Data_Types::ParameterDirectionKind>.


=head2 C<Foundation::Data_Types::ProcedureExpression>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::ProcedureExpression|UMMF::UML::MetaModel::Foundation::Data_Types::ProcedureExpression>.


=head2 C<Foundation::Data_Types::PseudostateKind>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::PseudostateKind|UMMF::UML::MetaModel::Foundation::Data_Types::PseudostateKind>.


=head2 C<Foundation::Data_Types::Real>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::Real|UMMF::UML::MetaModel::Foundation::Data_Types::Real>.


=head2 C<Foundation::Data_Types::ScopeKind>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::ScopeKind|UMMF::UML::MetaModel::Foundation::Data_Types::ScopeKind>.


=head2 C<Foundation::Data_Types::String>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::String|UMMF::UML::MetaModel::Foundation::Data_Types::String>.


=head2 C<Foundation::Data_Types::TimeExpression>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::TimeExpression|UMMF::UML::MetaModel::Foundation::Data_Types::TimeExpression>.


=head2 C<Foundation::Data_Types::TypeExpression>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::TypeExpression|UMMF::UML::MetaModel::Foundation::Data_Types::TypeExpression>.


=head2 C<Foundation::Data_Types::UnlimitedInteger>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::UnlimitedInteger|UMMF::UML::MetaModel::Foundation::Data_Types::UnlimitedInteger>.


=head2 C<Foundation::Data_Types::VisibilityKind>

See L<UMMF::UML::MetaModel::Foundation::Data_Types::VisibilityKind|UMMF::UML::MetaModel::Foundation::Data_Types::VisibilityKind>.


=head2 C<Foundation::Extension_Mechanisms::Stereotype>

See L<UMMF::UML::MetaModel::Foundation::Extension_Mechanisms::Stereotype|UMMF::UML::MetaModel::Foundation::Extension_Mechanisms::Stereotype>.


=head2 C<Foundation::Extension_Mechanisms::TagDefinition>

See L<UMMF::UML::MetaModel::Foundation::Extension_Mechanisms::TagDefinition|UMMF::UML::MetaModel::Foundation::Extension_Mechanisms::TagDefinition>.


=head2 C<Foundation::Extension_Mechanisms::TaggedValue>

See L<UMMF::UML::MetaModel::Foundation::Extension_Mechanisms::TaggedValue|UMMF::UML::MetaModel::Foundation::Extension_Mechanisms::TaggedValue>.


=head2 C<Model_Management::ElementImport>

See L<UMMF::UML::MetaModel::Model_Management::ElementImport|UMMF::UML::MetaModel::Model_Management::ElementImport>.


=head2 C<Model_Management::Model>

See L<UMMF::UML::MetaModel::Model_Management::Model|UMMF::UML::MetaModel::Model_Management::Model>.


=head2 C<Model_Management::Package>

See L<UMMF::UML::MetaModel::Model_Management::Package|UMMF::UML::MetaModel::Model_Management::Package>.


=head2 C<Model_Management::Subsystem>

See L<UMMF::UML::MetaModel::Model_Management::Subsystem|UMMF::UML::MetaModel::Model_Management::Subsystem>.


=head2 C<OCL::Expressions::AssociationClassEndCallExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::AssociationClassEndCallExp|UMMF::UML::MetaModel::OCL::Expressions::AssociationClassEndCallExp>.


=head2 C<OCL::Expressions::AssociationEndCallExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::AssociationEndCallExp|UMMF::UML::MetaModel::OCL::Expressions::AssociationEndCallExp>.


=head2 C<OCL::Expressions::AttributeCallExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::AttributeCallExp|UMMF::UML::MetaModel::OCL::Expressions::AttributeCallExp>.


=head2 C<OCL::Expressions::BooleanLiteralExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::BooleanLiteralExp|UMMF::UML::MetaModel::OCL::Expressions::BooleanLiteralExp>.


=head2 C<OCL::Expressions::CallAction>

See L<UMMF::UML::MetaModel::OCL::Expressions::CallAction|UMMF::UML::MetaModel::OCL::Expressions::CallAction>.


=head2 C<OCL::Expressions::CollectionItem>

See L<UMMF::UML::MetaModel::OCL::Expressions::CollectionItem|UMMF::UML::MetaModel::OCL::Expressions::CollectionItem>.


=head2 C<OCL::Expressions::CollectionKind>

See L<UMMF::UML::MetaModel::OCL::Expressions::CollectionKind|UMMF::UML::MetaModel::OCL::Expressions::CollectionKind>.


=head2 C<OCL::Expressions::CollectionLiteralExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::CollectionLiteralExp|UMMF::UML::MetaModel::OCL::Expressions::CollectionLiteralExp>.


=head2 C<OCL::Expressions::CollectionLiteralPart>

See L<UMMF::UML::MetaModel::OCL::Expressions::CollectionLiteralPart|UMMF::UML::MetaModel::OCL::Expressions::CollectionLiteralPart>.


=head2 C<OCL::Expressions::CollectionRange>

See L<UMMF::UML::MetaModel::OCL::Expressions::CollectionRange|UMMF::UML::MetaModel::OCL::Expressions::CollectionRange>.


=head2 C<OCL::Expressions::EnumLiteralExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::EnumLiteralExp|UMMF::UML::MetaModel::OCL::Expressions::EnumLiteralExp>.


=head2 C<OCL::Expressions::IfExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::IfExp|UMMF::UML::MetaModel::OCL::Expressions::IfExp>.


=head2 C<OCL::Expressions::IntegerLiteralExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::IntegerLiteralExp|UMMF::UML::MetaModel::OCL::Expressions::IntegerLiteralExp>.


=head2 C<OCL::Expressions::IterateExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::IterateExp|UMMF::UML::MetaModel::OCL::Expressions::IterateExp>.


=head2 C<OCL::Expressions::IteratorExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::IteratorExp|UMMF::UML::MetaModel::OCL::Expressions::IteratorExp>.


=head2 C<OCL::Expressions::LetExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::LetExp|UMMF::UML::MetaModel::OCL::Expressions::LetExp>.


=head2 C<OCL::Expressions::LiteralExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::LiteralExp|UMMF::UML::MetaModel::OCL::Expressions::LiteralExp>.


=head2 C<OCL::Expressions::LoopExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::LoopExp|UMMF::UML::MetaModel::OCL::Expressions::LoopExp>.


=head2 C<OCL::Expressions::ModelPropertyCallExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::ModelPropertyCallExp|UMMF::UML::MetaModel::OCL::Expressions::ModelPropertyCallExp>.


=head2 C<OCL::Expressions::NavigationCallExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::NavigationCallExp|UMMF::UML::MetaModel::OCL::Expressions::NavigationCallExp>.


=head2 C<OCL::Expressions::NumericLiteralExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::NumericLiteralExp|UMMF::UML::MetaModel::OCL::Expressions::NumericLiteralExp>.


=head2 C<OCL::Expressions::OclExpression>

See L<UMMF::UML::MetaModel::OCL::Expressions::OclExpression|UMMF::UML::MetaModel::OCL::Expressions::OclExpression>.


=head2 C<OCL::Expressions::OclMessageArg>

See L<UMMF::UML::MetaModel::OCL::Expressions::OclMessageArg|UMMF::UML::MetaModel::OCL::Expressions::OclMessageArg>.


=head2 C<OCL::Expressions::OclMessageExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::OclMessageExp|UMMF::UML::MetaModel::OCL::Expressions::OclMessageExp>.


=head2 C<OCL::Expressions::OperationCallExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::OperationCallExp|UMMF::UML::MetaModel::OCL::Expressions::OperationCallExp>.


=head2 C<OCL::Expressions::PrimitiveLiteralExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::PrimitiveLiteralExp|UMMF::UML::MetaModel::OCL::Expressions::PrimitiveLiteralExp>.


=head2 C<OCL::Expressions::PropertyCallExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::PropertyCallExp|UMMF::UML::MetaModel::OCL::Expressions::PropertyCallExp>.


=head2 C<OCL::Expressions::RealLiteralExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::RealLiteralExp|UMMF::UML::MetaModel::OCL::Expressions::RealLiteralExp>.


=head2 C<OCL::Expressions::SendAction>

See L<UMMF::UML::MetaModel::OCL::Expressions::SendAction|UMMF::UML::MetaModel::OCL::Expressions::SendAction>.


=head2 C<OCL::Expressions::StringLiteralExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::StringLiteralExp|UMMF::UML::MetaModel::OCL::Expressions::StringLiteralExp>.


=head2 C<OCL::Expressions::TupleLiteralExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::TupleLiteralExp|UMMF::UML::MetaModel::OCL::Expressions::TupleLiteralExp>.


=head2 C<OCL::Expressions::UnspecifiedValueExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::UnspecifiedValueExp|UMMF::UML::MetaModel::OCL::Expressions::UnspecifiedValueExp>.


=head2 C<OCL::Expressions::VariableDeclaration>

See L<UMMF::UML::MetaModel::OCL::Expressions::VariableDeclaration|UMMF::UML::MetaModel::OCL::Expressions::VariableDeclaration>.


=head2 C<OCL::Expressions::VariableExp>

See L<UMMF::UML::MetaModel::OCL::Expressions::VariableExp|UMMF::UML::MetaModel::OCL::Expressions::VariableExp>.


=head2 C<OCL::Types::BagType>

See L<UMMF::UML::MetaModel::OCL::Types::BagType|UMMF::UML::MetaModel::OCL::Types::BagType>.


=head2 C<OCL::Types::CollectionType>

See L<UMMF::UML::MetaModel::OCL::Types::CollectionType|UMMF::UML::MetaModel::OCL::Types::CollectionType>.


=head2 C<OCL::Types::OCLModelElementType>

See L<UMMF::UML::MetaModel::OCL::Types::OCLModelElementType|UMMF::UML::MetaModel::OCL::Types::OCLModelElementType>.


=head2 C<OCL::Types::OclMessageType>

See L<UMMF::UML::MetaModel::OCL::Types::OclMessageType|UMMF::UML::MetaModel::OCL::Types::OclMessageType>.


=head2 C<OCL::Types::OrdereredSetType>

See L<UMMF::UML::MetaModel::OCL::Types::OrdereredSetType|UMMF::UML::MetaModel::OCL::Types::OrdereredSetType>.


=head2 C<OCL::Types::SequenceType>

See L<UMMF::UML::MetaModel::OCL::Types::SequenceType|UMMF::UML::MetaModel::OCL::Types::SequenceType>.


=head2 C<OCL::Types::SetType>

See L<UMMF::UML::MetaModel::OCL::Types::SetType|UMMF::UML::MetaModel::OCL::Types::SetType>.


=head2 C<OCL::Types::TupleType>

See L<UMMF::UML::MetaModel::OCL::Types::TupleType|UMMF::UML::MetaModel::OCL::Types::TupleType>.


=head2 C<OCL::Types::VoidType>

See L<UMMF::UML::MetaModel::OCL::Types::VoidType|UMMF::UML::MetaModel::OCL::Types::VoidType>.


=head1 SEE ALSO

L<UMMF::UML::MetaModel|UMMF::UML::MetaModel>

=head1 VERSION

$Revision: 1.74 $

=head1 METHODS

=cut

#################################################################
# Supers
#


#################################################################
# Dependencies
#

use Carp qw(croak confess);


#################################################################
# Import them all!!!
# [ % model_package_use % ];
#


#################################################################
# Methods
#

my $factory_map = 
[
      'Collaboration',
      'UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::Collaboration',
      'Behavioral_Elements::Collaborations::Collaboration',
      'UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::Collaboration',
      'InteractionInstanceSet',
      'UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::InteractionInstanceSet',
      'Behavioral_Elements::Collaborations::InteractionInstanceSet',
      'UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::InteractionInstanceSet',
      'AssociationRole',
      'UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::AssociationRole',
      'Behavioral_Elements::Collaborations::AssociationRole',
      'UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::AssociationRole',
      'AssociationEndRole',
      'UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::AssociationEndRole',
      'Behavioral_Elements::Collaborations::AssociationEndRole',
      'UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::AssociationEndRole',
      'Interaction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::Interaction',
      'Behavioral_Elements::Collaborations::Interaction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::Interaction',
      'CollaborationInstanceSet',
      'UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::CollaborationInstanceSet',
      'Behavioral_Elements::Collaborations::CollaborationInstanceSet',
      'UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::CollaborationInstanceSet',
      'Message',
      'UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::Message',
      'Behavioral_Elements::Collaborations::Message',
      'UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::Message',
      'ClassifierRole',
      'UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::ClassifierRole',
      'Behavioral_Elements::Collaborations::ClassifierRole',
      'UMMF::UML::MetaModel::Behavioral_Elements::Collaborations::ClassifierRole',
      'LinkEnd',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::LinkEnd',
      'Behavioral_Elements::Common_Behavior::LinkEnd',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::LinkEnd',
      'ComponentInstance',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::ComponentInstance',
      'Behavioral_Elements::Common_Behavior::ComponentInstance',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::ComponentInstance',
      'Procedure',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Procedure',
      'Behavioral_Elements::Common_Behavior::Procedure',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Procedure',
      'LinkObject',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::LinkObject',
      'Behavioral_Elements::Common_Behavior::LinkObject',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::LinkObject',
      'Reception',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Reception',
      'Behavioral_Elements::Common_Behavior::Reception',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Reception',
      'Signal',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Signal',
      'Behavioral_Elements::Common_Behavior::Signal',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Signal',
      'Exception',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Exception',
      'Behavioral_Elements::Common_Behavior::Exception',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Exception',
      'Stimulus',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Stimulus',
      'Behavioral_Elements::Common_Behavior::Stimulus',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Stimulus',
      'Link',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Link',
      'Behavioral_Elements::Common_Behavior::Link',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Link',
      'SubsystemInstance',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::SubsystemInstance',
      'Behavioral_Elements::Common_Behavior::SubsystemInstance',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::SubsystemInstance',
      'Object',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Object',
      'Behavioral_Elements::Common_Behavior::Object',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Object',
      'Instance',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Instance',
      'Behavioral_Elements::Common_Behavior::Instance',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::Instance',
      'DataValue',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::DataValue',
      'Behavioral_Elements::Common_Behavior::DataValue',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::DataValue',
      'NodeInstance',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::NodeInstance',
      'Behavioral_Elements::Common_Behavior::NodeInstance',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::NodeInstance',
      'AttributeLink',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::AttributeLink',
      'Behavioral_Elements::Common_Behavior::AttributeLink',
      'UMMF::UML::MetaModel::Behavioral_Elements::Common_Behavior::AttributeLink',
      'Include',
      'UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::Include',
      'Behavioral_Elements::Use_Cases::Include',
      'UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::Include',
      'Extend',
      'UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::Extend',
      'Behavioral_Elements::Use_Cases::Extend',
      'UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::Extend',
      'UseCase',
      'UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::UseCase',
      'Behavioral_Elements::Use_Cases::UseCase',
      'UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::UseCase',
      'UseCaseInstance',
      'UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::UseCaseInstance',
      'Behavioral_Elements::Use_Cases::UseCaseInstance',
      'UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::UseCaseInstance',
      'ExtensionPoint',
      'UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::ExtensionPoint',
      'Behavioral_Elements::Use_Cases::ExtensionPoint',
      'UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::ExtensionPoint',
      'Actor',
      'UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::Actor',
      'Behavioral_Elements::Use_Cases::Actor',
      'UMMF::UML::MetaModel::Behavioral_Elements::Use_Cases::Actor',
      'ClassifierInState',
      'UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::ClassifierInState',
      'Behavioral_Elements::Activity_Graphs::ClassifierInState',
      'UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::ClassifierInState',
      'ActionState',
      'UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::ActionState',
      'Behavioral_Elements::Activity_Graphs::ActionState',
      'UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::ActionState',
      'ActivityGraph',
      'UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::ActivityGraph',
      'Behavioral_Elements::Activity_Graphs::ActivityGraph',
      'UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::ActivityGraph',
      'Partition',
      'UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::Partition',
      'Behavioral_Elements::Activity_Graphs::Partition',
      'UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::Partition',
      'SubactivityState',
      'UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::SubactivityState',
      'Behavioral_Elements::Activity_Graphs::SubactivityState',
      'UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::SubactivityState',
      'ObjectFlowState',
      'UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::ObjectFlowState',
      'Behavioral_Elements::Activity_Graphs::ObjectFlowState',
      'UMMF::UML::MetaModel::Behavioral_Elements::Activity_Graphs::ObjectFlowState',
      'SignalEvent',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::SignalEvent',
      'Behavioral_Elements::State_Machines::SignalEvent',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::SignalEvent',
      'ChangeEvent',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::ChangeEvent',
      'Behavioral_Elements::State_Machines::ChangeEvent',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::ChangeEvent',
      'SubmachineState',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::SubmachineState',
      'Behavioral_Elements::State_Machines::SubmachineState',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::SubmachineState',
      'CompositeState',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::CompositeState',
      'Behavioral_Elements::State_Machines::CompositeState',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::CompositeState',
      'SynchState',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::SynchState',
      'Behavioral_Elements::State_Machines::SynchState',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::SynchState',
      'StubState',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::StubState',
      'Behavioral_Elements::State_Machines::StubState',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::StubState',
      'Pseudostate',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::Pseudostate',
      'Behavioral_Elements::State_Machines::Pseudostate',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::Pseudostate',
      'CallEvent',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::CallEvent',
      'Behavioral_Elements::State_Machines::CallEvent',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::CallEvent',
      'Transition',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::Transition',
      'Behavioral_Elements::State_Machines::Transition',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::Transition',
      'StateVertex',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::StateVertex',
      'Behavioral_Elements::State_Machines::StateVertex',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::StateVertex',
      'Guard',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::Guard',
      'Behavioral_Elements::State_Machines::Guard',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::Guard',
      'Event',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::Event',
      'Behavioral_Elements::State_Machines::Event',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::Event',
      'State',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::State',
      'Behavioral_Elements::State_Machines::State',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::State',
      'StateMachine',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::StateMachine',
      'Behavioral_Elements::State_Machines::StateMachine',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::StateMachine',
      'FinalState',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::FinalState',
      'Behavioral_Elements::State_Machines::FinalState',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::FinalState',
      'SimpleState',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::SimpleState',
      'Behavioral_Elements::State_Machines::SimpleState',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::SimpleState',
      'TimeEvent',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::TimeEvent',
      'Behavioral_Elements::State_Machines::TimeEvent',
      'UMMF::UML::MetaModel::Behavioral_Elements::State_Machines::TimeEvent',
      'DataFlow',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow',
      'Behavioral_Elements::Actions::Action_Foundation::DataFlow',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow',
      'OutputPin',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin',
      'Behavioral_Elements::Actions::Action_Foundation::OutputPin',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin',
      'Action',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action',
      'Behavioral_Elements::Actions::Action_Foundation::Action',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action',
      'Pin',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Pin',
      'Behavioral_Elements::Actions::Action_Foundation::Pin',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Pin',
      'Variable',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Variable',
      'Behavioral_Elements::Actions::Action_Foundation::Variable',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Variable',
      'InputPin',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::InputPin',
      'Behavioral_Elements::Actions::Action_Foundation::InputPin',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::InputPin',
      'PrimitiveAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::PrimitiveAction',
      'Behavioral_Elements::Actions::Action_Foundation::PrimitiveAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::PrimitiveAction',
      'Procedure',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure',
      'Behavioral_Elements::Actions::Action_Foundation::Procedure',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure',
      'ControlFlow',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::ControlFlow',
      'Behavioral_Elements::Actions::Action_Foundation::ControlFlow',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::ControlFlow',
      'PrimitiveFunction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::PrimitiveFunction',
      'Behavioral_Elements::Actions::Computation_Actions::PrimitiveFunction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::PrimitiveFunction',
      'UnmarshalAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::UnmarshalAction',
      'Behavioral_Elements::Actions::Computation_Actions::UnmarshalAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::UnmarshalAction',
      'MarshalAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::MarshalAction',
      'Behavioral_Elements::Actions::Computation_Actions::MarshalAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::MarshalAction',
      'ApplyFunctionAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::ApplyFunctionAction',
      'Behavioral_Elements::Actions::Computation_Actions::ApplyFunctionAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::ApplyFunctionAction',
      'TestIdentifyAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::TestIdentifyAction',
      'Behavioral_Elements::Actions::Computation_Actions::TestIdentifyAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::TestIdentifyAction',
      'CodeAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::CodeAction',
      'Behavioral_Elements::Actions::Computation_Actions::CodeAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::CodeAction',
      'LiteralValueAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::LiteralValueAction',
      'Behavioral_Elements::Actions::Computation_Actions::LiteralValueAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::LiteralValueAction',
      'ArgumentSpecification',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::ArgumentSpecification',
      'Behavioral_Elements::Actions::Computation_Actions::ArgumentSpecification',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::ArgumentSpecification',
      'NullAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::NullAction',
      'Behavioral_Elements::Actions::Computation_Actions::NullAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Computation_Actions::NullAction',
      'ConditionalAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::ConditionalAction',
      'Behavioral_Elements::Actions::Composite_Actions::ConditionalAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::ConditionalAction',
      'Clause',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::Clause',
      'Behavioral_Elements::Actions::Composite_Actions::Clause',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::Clause',
      'GroupAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::GroupAction',
      'Behavioral_Elements::Actions::Composite_Actions::GroupAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::GroupAction',
      'LoopAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction',
      'Behavioral_Elements::Actions::Composite_Actions::LoopAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction',
      'ReadExtentAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::ReadExtentAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::ReadExtentAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::ReadExtentAction',
      'CallProcedureAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::CallProcedureAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::CallProcedureAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::CallProcedureAction',
      'StartObjectStateMachineAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::StartObjectStateMachineAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::StartObjectStateMachineAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::StartObjectStateMachineAction',
      'ReadSelfAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::ReadSelfAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::ReadSelfAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Other_Actions::ReadSelfAction',
      'DestroyObjectAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::DestroyObjectAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::DestroyObjectAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::DestroyObjectAction',
      'ReadIsClassifiedObjectAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::ReadIsClassifiedObjectAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::ReadIsClassifiedObjectAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::ReadIsClassifiedObjectAction',
      'CreateObjectAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::CreateObjectAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::CreateObjectAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::CreateObjectAction',
      'ReclassifyObjectAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::ReclassifyObjectAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::ReclassifyObjectAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Object_Actions::ReclassifyObjectAction',
      'DestroyLinkAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::DestroyLinkAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::DestroyLinkAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::DestroyLinkAction',
      'CreateLinkObjectAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::CreateLinkObjectAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::CreateLinkObjectAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::CreateLinkObjectAction',
      'ReadLinkAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkAction',
      'ReadLinkObjectQualifierAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkObjectQualifierAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkObjectQualifierAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkObjectQualifierAction',
      'QualifierValue',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::QualifierValue',
      'Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::QualifierValue',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::QualifierValue',
      'ReadLinkObjectEndAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkObjectEndAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkObjectEndAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ReadLinkObjectEndAction',
      'LinkEndCreationData',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkEndCreationData',
      'Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkEndCreationData',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkEndCreationData',
      'CreateLinkAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::CreateLinkAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::CreateLinkAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::CreateLinkAction',
      'WriteLinkAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::WriteLinkAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::WriteLinkAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::WriteLinkAction',
      'LinkEndData',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkEndData',
      'Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkEndData',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkEndData',
      'ClearAssociationAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ClearAssociationAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ClearAssociationAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::ClearAssociationAction',
      'LinkAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Association_Actions::LinkAction',
      'ReadAttributeAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::ReadAttributeAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::ReadAttributeAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::ReadAttributeAction',
      'WriteAttributeAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::WriteAttributeAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::WriteAttributeAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::WriteAttributeAction',
      'AttributeAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::AttributeAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::AttributeAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::AttributeAction',
      'AddAttributeValueAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::AddAttributeValueAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::AddAttributeValueAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::AddAttributeValueAction',
      'RemoveAttributeValueAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::RemoveAttributeValueAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::RemoveAttributeValueAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::RemoveAttributeValueAction',
      'ClearAttributeAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::ClearAttributeAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::ClearAttributeAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Attribute_Actions::ClearAttributeAction',
      'WriteVariableAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::WriteVariableAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::WriteVariableAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::WriteVariableAction',
      'AddVariableValueAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::AddVariableValueAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::AddVariableValueAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::AddVariableValueAction',
      'ReadVariableAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::ReadVariableAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::ReadVariableAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::ReadVariableAction',
      'RemoveVariableValueAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::RemoveVariableValueAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::RemoveVariableValueAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::RemoveVariableValueAction',
      'VariableAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::VariableAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::VariableAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::VariableAction',
      'ClearVariableAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::ClearVariableAction',
      'Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::ClearVariableAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Read_Write_Actions::Variable_Actions::ClearVariableAction',
      'SynchronousInvocationAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::SynchronousInvocationAction',
      'Behavioral_Elements::Actions::Messaging_Actions::SynchronousInvocationAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::SynchronousInvocationAction',
      'AsynchronousInvocationAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::AsynchronousInvocationAction',
      'Behavioral_Elements::Actions::Messaging_Actions::AsynchronousInvocationAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::AsynchronousInvocationAction',
      'CallOperationAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::CallOperationAction',
      'Behavioral_Elements::Actions::Messaging_Actions::CallOperationAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::CallOperationAction',
      'InvocationAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::InvocationAction',
      'Behavioral_Elements::Actions::Messaging_Actions::InvocationAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::InvocationAction',
      'SendSignalAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::SendSignalAction',
      'Behavioral_Elements::Actions::Messaging_Actions::SendSignalAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::SendSignalAction',
      'BroadcastSignalAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::BroadcastSignalAction',
      'Behavioral_Elements::Actions::Messaging_Actions::BroadcastSignalAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::BroadcastSignalAction',
      'ExplicitInvocationAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::ExplicitInvocationAction',
      'Behavioral_Elements::Actions::Messaging_Actions::ExplicitInvocationAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Messaging_Actions::ExplicitInvocationAction',
      'ReduceAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Reduce::ReduceAction',
      'Behavioral_Elements::Actions::Collection_Actions::Reduce::ReduceAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Reduce::ReduceAction',
      'FilterAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Filter::FilterAction',
      'Behavioral_Elements::Actions::Collection_Actions::Filter::FilterAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Filter::FilterAction',
      'CollectionAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::CollectionAction',
      'Behavioral_Elements::Actions::Collection_Actions::CollectionAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::CollectionAction',
      'MapAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Map::MapAction',
      'Behavioral_Elements::Actions::Collection_Actions::Map::MapAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Map::MapAction',
      'IterateAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Iterate::IterateAction',
      'Behavioral_Elements::Actions::Collection_Actions::Iterate::IterateAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Iterate::IterateAction',
      'JumpAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Jump_Actions::JumpAction',
      'Behavioral_Elements::Actions::Jump_Actions::JumpAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Jump_Actions::JumpAction',
      'JumpHandler',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Jump_Actions::JumpHandler',
      'Behavioral_Elements::Actions::Jump_Actions::JumpHandler',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Jump_Actions::JumpHandler',
      'HandlerAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Jump_Actions::HandlerAction',
      'Behavioral_Elements::Actions::Jump_Actions::HandlerAction',
      'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Jump_Actions::HandlerAction',
      'Name',
      'UMMF::UML::MetaModel::Foundation::Data_Types::Name',
      'Foundation::Data_Types::Name',
      'UMMF::UML::MetaModel::Foundation::Data_Types::Name',
      'Multiplicity',
      'UMMF::UML::MetaModel::Foundation::Data_Types::Multiplicity',
      'Foundation::Data_Types::Multiplicity',
      'UMMF::UML::MetaModel::Foundation::Data_Types::Multiplicity',
      'AggregationKind',
      'UMMF::UML::MetaModel::Foundation::Data_Types::AggregationKind',
      'Foundation::Data_Types::AggregationKind',
      'UMMF::UML::MetaModel::Foundation::Data_Types::AggregationKind',
      'BooleanExpression',
      'UMMF::UML::MetaModel::Foundation::Data_Types::BooleanExpression',
      'Foundation::Data_Types::BooleanExpression',
      'UMMF::UML::MetaModel::Foundation::Data_Types::BooleanExpression',
      'ArgListExpression',
      'UMMF::UML::MetaModel::Foundation::Data_Types::ArgListExpression',
      'Foundation::Data_Types::ArgListExpression',
      'UMMF::UML::MetaModel::Foundation::Data_Types::ArgListExpression',
      'ChangeableKind',
      'UMMF::UML::MetaModel::Foundation::Data_Types::ChangeableKind',
      'Foundation::Data_Types::ChangeableKind',
      'UMMF::UML::MetaModel::Foundation::Data_Types::ChangeableKind',
      'Integer',
      'UMMF::UML::MetaModel::Foundation::Data_Types::Integer',
      'Foundation::Data_Types::Integer',
      'UMMF::UML::MetaModel::Foundation::Data_Types::Integer',
      'Geometry',
      'UMMF::UML::MetaModel::Foundation::Data_Types::Geometry',
      'Foundation::Data_Types::Geometry',
      'UMMF::UML::MetaModel::Foundation::Data_Types::Geometry',
      'MappingExpression',
      'UMMF::UML::MetaModel::Foundation::Data_Types::MappingExpression',
      'Foundation::Data_Types::MappingExpression',
      'UMMF::UML::MetaModel::Foundation::Data_Types::MappingExpression',
      'Boolean',
      'UMMF::UML::MetaModel::Foundation::Data_Types::Boolean',
      'Foundation::Data_Types::Boolean',
      'UMMF::UML::MetaModel::Foundation::Data_Types::Boolean',
      'CallConcurrencyKind',
      'UMMF::UML::MetaModel::Foundation::Data_Types::CallConcurrencyKind',
      'Foundation::Data_Types::CallConcurrencyKind',
      'UMMF::UML::MetaModel::Foundation::Data_Types::CallConcurrencyKind',
      'ScopeKind',
      'UMMF::UML::MetaModel::Foundation::Data_Types::ScopeKind',
      'Foundation::Data_Types::ScopeKind',
      'UMMF::UML::MetaModel::Foundation::Data_Types::ScopeKind',
      'LocationReference',
      'UMMF::UML::MetaModel::Foundation::Data_Types::LocationReference',
      'Foundation::Data_Types::LocationReference',
      'UMMF::UML::MetaModel::Foundation::Data_Types::LocationReference',
      'PseudostateKind',
      'UMMF::UML::MetaModel::Foundation::Data_Types::PseudostateKind',
      'Foundation::Data_Types::PseudostateKind',
      'UMMF::UML::MetaModel::Foundation::Data_Types::PseudostateKind',
      'ProcedureExpression',
      'UMMF::UML::MetaModel::Foundation::Data_Types::ProcedureExpression',
      'Foundation::Data_Types::ProcedureExpression',
      'UMMF::UML::MetaModel::Foundation::Data_Types::ProcedureExpression',
      'UnlimitedInteger',
      'UMMF::UML::MetaModel::Foundation::Data_Types::UnlimitedInteger',
      'Foundation::Data_Types::UnlimitedInteger',
      'UMMF::UML::MetaModel::Foundation::Data_Types::UnlimitedInteger',
      'ParameterDirectionKind',
      'UMMF::UML::MetaModel::Foundation::Data_Types::ParameterDirectionKind',
      'Foundation::Data_Types::ParameterDirectionKind',
      'UMMF::UML::MetaModel::Foundation::Data_Types::ParameterDirectionKind',
      'MultiplicityRange',
      'UMMF::UML::MetaModel::Foundation::Data_Types::MultiplicityRange',
      'Foundation::Data_Types::MultiplicityRange',
      'UMMF::UML::MetaModel::Foundation::Data_Types::MultiplicityRange',
      'Real',
      'UMMF::UML::MetaModel::Foundation::Data_Types::Real',
      'Foundation::Data_Types::Real',
      'UMMF::UML::MetaModel::Foundation::Data_Types::Real',
      'TimeExpression',
      'UMMF::UML::MetaModel::Foundation::Data_Types::TimeExpression',
      'Foundation::Data_Types::TimeExpression',
      'UMMF::UML::MetaModel::Foundation::Data_Types::TimeExpression',
      'OrderingKind',
      'UMMF::UML::MetaModel::Foundation::Data_Types::OrderingKind',
      'Foundation::Data_Types::OrderingKind',
      'UMMF::UML::MetaModel::Foundation::Data_Types::OrderingKind',
      'VisibilityKind',
      'UMMF::UML::MetaModel::Foundation::Data_Types::VisibilityKind',
      'Foundation::Data_Types::VisibilityKind',
      'UMMF::UML::MetaModel::Foundation::Data_Types::VisibilityKind',
      'String',
      'UMMF::UML::MetaModel::Foundation::Data_Types::String',
      'Foundation::Data_Types::String',
      'UMMF::UML::MetaModel::Foundation::Data_Types::String',
      'Expression',
      'UMMF::UML::MetaModel::Foundation::Data_Types::Expression',
      'Foundation::Data_Types::Expression',
      'UMMF::UML::MetaModel::Foundation::Data_Types::Expression',
      'Mapping',
      'UMMF::UML::MetaModel::Foundation::Data_Types::Mapping',
      'Foundation::Data_Types::Mapping',
      'UMMF::UML::MetaModel::Foundation::Data_Types::Mapping',
      'TypeExpression',
      'UMMF::UML::MetaModel::Foundation::Data_Types::TypeExpression',
      'Foundation::Data_Types::TypeExpression',
      'UMMF::UML::MetaModel::Foundation::Data_Types::TypeExpression',
      'Stereotype',
      'UMMF::UML::MetaModel::Foundation::Extension_Mechanisms::Stereotype',
      'Foundation::Extension_Mechanisms::Stereotype',
      'UMMF::UML::MetaModel::Foundation::Extension_Mechanisms::Stereotype',
      'TagDefinition',
      'UMMF::UML::MetaModel::Foundation::Extension_Mechanisms::TagDefinition',
      'Foundation::Extension_Mechanisms::TagDefinition',
      'UMMF::UML::MetaModel::Foundation::Extension_Mechanisms::TagDefinition',
      'TaggedValue',
      'UMMF::UML::MetaModel::Foundation::Extension_Mechanisms::TaggedValue',
      'Foundation::Extension_Mechanisms::TaggedValue',
      'UMMF::UML::MetaModel::Foundation::Extension_Mechanisms::TaggedValue',
      'Constraint',
      'UMMF::UML::MetaModel::Foundation::Core::Constraint',
      'Foundation::Core::Constraint',
      'UMMF::UML::MetaModel::Foundation::Core::Constraint',
      'Generalization',
      'UMMF::UML::MetaModel::Foundation::Core::Generalization',
      'Foundation::Core::Generalization',
      'UMMF::UML::MetaModel::Foundation::Core::Generalization',
      'Primitive',
      'UMMF::UML::MetaModel::Foundation::Core::Primitive',
      'Foundation::Core::Primitive',
      'UMMF::UML::MetaModel::Foundation::Core::Primitive',
      'Parameter',
      'UMMF::UML::MetaModel::Foundation::Core::Parameter',
      'Foundation::Core::Parameter',
      'UMMF::UML::MetaModel::Foundation::Core::Parameter',
      'Operation',
      'UMMF::UML::MetaModel::Foundation::Core::Operation',
      'Foundation::Core::Operation',
      'UMMF::UML::MetaModel::Foundation::Core::Operation',
      'Dependency',
      'UMMF::UML::MetaModel::Foundation::Core::Dependency',
      'Foundation::Core::Dependency',
      'UMMF::UML::MetaModel::Foundation::Core::Dependency',
      'Comment',
      'UMMF::UML::MetaModel::Foundation::Core::Comment',
      'Foundation::Core::Comment',
      'UMMF::UML::MetaModel::Foundation::Core::Comment',
      'PresentationElement',
      'UMMF::UML::MetaModel::Foundation::Core::PresentationElement',
      'Foundation::Core::PresentationElement',
      'UMMF::UML::MetaModel::Foundation::Core::PresentationElement',
      'AssociationClass',
      'UMMF::UML::MetaModel::Foundation::Core::AssociationClass',
      'Foundation::Core::AssociationClass',
      'UMMF::UML::MetaModel::Foundation::Core::AssociationClass',
      'Component',
      'UMMF::UML::MetaModel::Foundation::Core::Component',
      'Foundation::Core::Component',
      'UMMF::UML::MetaModel::Foundation::Core::Component',
      'Relationship',
      'UMMF::UML::MetaModel::Foundation::Core::Relationship',
      'Foundation::Core::Relationship',
      'UMMF::UML::MetaModel::Foundation::Core::Relationship',
      'EnumerationLiteral',
      'UMMF::UML::MetaModel::Foundation::Core::EnumerationLiteral',
      'Foundation::Core::EnumerationLiteral',
      'UMMF::UML::MetaModel::Foundation::Core::EnumerationLiteral',
      'ModelElement',
      'UMMF::UML::MetaModel::Foundation::Core::ModelElement',
      'Foundation::Core::ModelElement',
      'UMMF::UML::MetaModel::Foundation::Core::ModelElement',
      'Method',
      'UMMF::UML::MetaModel::Foundation::Core::Method',
      'Foundation::Core::Method',
      'UMMF::UML::MetaModel::Foundation::Core::Method',
      'ProgrammingLanguageDataType',
      'UMMF::UML::MetaModel::Foundation::Core::ProgrammingLanguageDataType',
      'Foundation::Core::ProgrammingLanguageDataType',
      'UMMF::UML::MetaModel::Foundation::Core::ProgrammingLanguageDataType',
      'Enumeration',
      'UMMF::UML::MetaModel::Foundation::Core::Enumeration',
      'Foundation::Core::Enumeration',
      'UMMF::UML::MetaModel::Foundation::Core::Enumeration',
      'ElementOwnership',
      'UMMF::UML::MetaModel::Foundation::Core::ElementOwnership',
      'Foundation::Core::ElementOwnership',
      'UMMF::UML::MetaModel::Foundation::Core::ElementOwnership',
      'AssociationEnd',
      'UMMF::UML::MetaModel::Foundation::Core::AssociationEnd',
      'Foundation::Core::AssociationEnd',
      'UMMF::UML::MetaModel::Foundation::Core::AssociationEnd',
      'Association',
      'UMMF::UML::MetaModel::Foundation::Core::Association',
      'Foundation::Core::Association',
      'UMMF::UML::MetaModel::Foundation::Core::Association',
      'Flow',
      'UMMF::UML::MetaModel::Foundation::Core::Flow',
      'Foundation::Core::Flow',
      'UMMF::UML::MetaModel::Foundation::Core::Flow',
      'Artifact',
      'UMMF::UML::MetaModel::Foundation::Core::Artifact',
      'Foundation::Core::Artifact',
      'UMMF::UML::MetaModel::Foundation::Core::Artifact',
      'DataType',
      'UMMF::UML::MetaModel::Foundation::Core::DataType',
      'Foundation::Core::DataType',
      'UMMF::UML::MetaModel::Foundation::Core::DataType',
      'Namespace',
      'UMMF::UML::MetaModel::Foundation::Core::Namespace',
      'Foundation::Core::Namespace',
      'UMMF::UML::MetaModel::Foundation::Core::Namespace',
      'Element',
      'UMMF::UML::MetaModel::Foundation::Core::Element',
      'Foundation::Core::Element',
      'UMMF::UML::MetaModel::Foundation::Core::Element',
      'TemplateParameter',
      'UMMF::UML::MetaModel::Foundation::Core::TemplateParameter',
      'Foundation::Core::TemplateParameter',
      'UMMF::UML::MetaModel::Foundation::Core::TemplateParameter',
      'Usage',
      'UMMF::UML::MetaModel::Foundation::Core::Usage',
      'Foundation::Core::Usage',
      'UMMF::UML::MetaModel::Foundation::Core::Usage',
      'GeneralizableElement',
      'UMMF::UML::MetaModel::Foundation::Core::GeneralizableElement',
      'Foundation::Core::GeneralizableElement',
      'UMMF::UML::MetaModel::Foundation::Core::GeneralizableElement',
      'Permission',
      'UMMF::UML::MetaModel::Foundation::Core::Permission',
      'Foundation::Core::Permission',
      'UMMF::UML::MetaModel::Foundation::Core::Permission',
      'Classifier',
      'UMMF::UML::MetaModel::Foundation::Core::Classifier',
      'Foundation::Core::Classifier',
      'UMMF::UML::MetaModel::Foundation::Core::Classifier',
      'Node',
      'UMMF::UML::MetaModel::Foundation::Core::Node',
      'Foundation::Core::Node',
      'UMMF::UML::MetaModel::Foundation::Core::Node',
      'TemplateArgument',
      'UMMF::UML::MetaModel::Foundation::Core::TemplateArgument',
      'Foundation::Core::TemplateArgument',
      'UMMF::UML::MetaModel::Foundation::Core::TemplateArgument',
      'Abstraction',
      'UMMF::UML::MetaModel::Foundation::Core::Abstraction',
      'Foundation::Core::Abstraction',
      'UMMF::UML::MetaModel::Foundation::Core::Abstraction',
      'Interface',
      'UMMF::UML::MetaModel::Foundation::Core::Interface',
      'Foundation::Core::Interface',
      'UMMF::UML::MetaModel::Foundation::Core::Interface',
      'BehavioralFeature',
      'UMMF::UML::MetaModel::Foundation::Core::BehavioralFeature',
      'Foundation::Core::BehavioralFeature',
      'UMMF::UML::MetaModel::Foundation::Core::BehavioralFeature',
      'StructuralFeature',
      'UMMF::UML::MetaModel::Foundation::Core::StructuralFeature',
      'Foundation::Core::StructuralFeature',
      'UMMF::UML::MetaModel::Foundation::Core::StructuralFeature',
      'Feature',
      'UMMF::UML::MetaModel::Foundation::Core::Feature',
      'Foundation::Core::Feature',
      'UMMF::UML::MetaModel::Foundation::Core::Feature',
      'Binding',
      'UMMF::UML::MetaModel::Foundation::Core::Binding',
      'Foundation::Core::Binding',
      'UMMF::UML::MetaModel::Foundation::Core::Binding',
      'Attribute',
      'UMMF::UML::MetaModel::Foundation::Core::Attribute',
      'Foundation::Core::Attribute',
      'UMMF::UML::MetaModel::Foundation::Core::Attribute',
      'Class',
      'UMMF::UML::MetaModel::Foundation::Core::Class',
      'Foundation::Core::Class',
      'UMMF::UML::MetaModel::Foundation::Core::Class',
      'OrdereredSetType',
      'UMMF::UML::MetaModel::OCL::Types::OrdereredSetType',
      'OCL::Types::OrdereredSetType',
      'UMMF::UML::MetaModel::OCL::Types::OrdereredSetType',
      'SequenceType',
      'UMMF::UML::MetaModel::OCL::Types::SequenceType',
      'OCL::Types::SequenceType',
      'UMMF::UML::MetaModel::OCL::Types::SequenceType',
      'BagType',
      'UMMF::UML::MetaModel::OCL::Types::BagType',
      'OCL::Types::BagType',
      'UMMF::UML::MetaModel::OCL::Types::BagType',
      'TupleType',
      'UMMF::UML::MetaModel::OCL::Types::TupleType',
      'OCL::Types::TupleType',
      'UMMF::UML::MetaModel::OCL::Types::TupleType',
      'SetType',
      'UMMF::UML::MetaModel::OCL::Types::SetType',
      'OCL::Types::SetType',
      'UMMF::UML::MetaModel::OCL::Types::SetType',
      'VoidType',
      'UMMF::UML::MetaModel::OCL::Types::VoidType',
      'OCL::Types::VoidType',
      'UMMF::UML::MetaModel::OCL::Types::VoidType',
      'CollectionType',
      'UMMF::UML::MetaModel::OCL::Types::CollectionType',
      'OCL::Types::CollectionType',
      'UMMF::UML::MetaModel::OCL::Types::CollectionType',
      'OCLModelElementType',
      'UMMF::UML::MetaModel::OCL::Types::OCLModelElementType',
      'OCL::Types::OCLModelElementType',
      'UMMF::UML::MetaModel::OCL::Types::OCLModelElementType',
      'OclMessageType',
      'UMMF::UML::MetaModel::OCL::Types::OclMessageType',
      'OCL::Types::OclMessageType',
      'UMMF::UML::MetaModel::OCL::Types::OclMessageType',
      'CollectionLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::CollectionLiteralExp',
      'OCL::Expressions::CollectionLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::CollectionLiteralExp',
      'LoopExp',
      'UMMF::UML::MetaModel::OCL::Expressions::LoopExp',
      'OCL::Expressions::LoopExp',
      'UMMF::UML::MetaModel::OCL::Expressions::LoopExp',
      'CallAction',
      'UMMF::UML::MetaModel::OCL::Expressions::CallAction',
      'OCL::Expressions::CallAction',
      'UMMF::UML::MetaModel::OCL::Expressions::CallAction',
      'EnumLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::EnumLiteralExp',
      'OCL::Expressions::EnumLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::EnumLiteralExp',
      'CollectionLiteralPart',
      'UMMF::UML::MetaModel::OCL::Expressions::CollectionLiteralPart',
      'OCL::Expressions::CollectionLiteralPart',
      'UMMF::UML::MetaModel::OCL::Expressions::CollectionLiteralPart',
      'UnspecifiedValueExp',
      'UMMF::UML::MetaModel::OCL::Expressions::UnspecifiedValueExp',
      'OCL::Expressions::UnspecifiedValueExp',
      'UMMF::UML::MetaModel::OCL::Expressions::UnspecifiedValueExp',
      'RealLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::RealLiteralExp',
      'OCL::Expressions::RealLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::RealLiteralExp',
      'TupleLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::TupleLiteralExp',
      'OCL::Expressions::TupleLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::TupleLiteralExp',
      'CollectionRange',
      'UMMF::UML::MetaModel::OCL::Expressions::CollectionRange',
      'OCL::Expressions::CollectionRange',
      'UMMF::UML::MetaModel::OCL::Expressions::CollectionRange',
      'NumericLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::NumericLiteralExp',
      'OCL::Expressions::NumericLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::NumericLiteralExp',
      'IterateExp',
      'UMMF::UML::MetaModel::OCL::Expressions::IterateExp',
      'OCL::Expressions::IterateExp',
      'UMMF::UML::MetaModel::OCL::Expressions::IterateExp',
      'NavigationCallExp',
      'UMMF::UML::MetaModel::OCL::Expressions::NavigationCallExp',
      'OCL::Expressions::NavigationCallExp',
      'UMMF::UML::MetaModel::OCL::Expressions::NavigationCallExp',
      'IteratorExp',
      'UMMF::UML::MetaModel::OCL::Expressions::IteratorExp',
      'OCL::Expressions::IteratorExp',
      'UMMF::UML::MetaModel::OCL::Expressions::IteratorExp',
      'StringLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::StringLiteralExp',
      'OCL::Expressions::StringLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::StringLiteralExp',
      'VariableDeclaration',
      'UMMF::UML::MetaModel::OCL::Expressions::VariableDeclaration',
      'OCL::Expressions::VariableDeclaration',
      'UMMF::UML::MetaModel::OCL::Expressions::VariableDeclaration',
      'AttributeCallExp',
      'UMMF::UML::MetaModel::OCL::Expressions::AttributeCallExp',
      'OCL::Expressions::AttributeCallExp',
      'UMMF::UML::MetaModel::OCL::Expressions::AttributeCallExp',
      'LetExp',
      'UMMF::UML::MetaModel::OCL::Expressions::LetExp',
      'OCL::Expressions::LetExp',
      'UMMF::UML::MetaModel::OCL::Expressions::LetExp',
      'LiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::LiteralExp',
      'OCL::Expressions::LiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::LiteralExp',
      'CollectionItem',
      'UMMF::UML::MetaModel::OCL::Expressions::CollectionItem',
      'OCL::Expressions::CollectionItem',
      'UMMF::UML::MetaModel::OCL::Expressions::CollectionItem',
      'CollectionKind',
      'UMMF::UML::MetaModel::OCL::Expressions::CollectionKind',
      'OCL::Expressions::CollectionKind',
      'UMMF::UML::MetaModel::OCL::Expressions::CollectionKind',
      'IfExp',
      'UMMF::UML::MetaModel::OCL::Expressions::IfExp',
      'OCL::Expressions::IfExp',
      'UMMF::UML::MetaModel::OCL::Expressions::IfExp',
      'PropertyCallExp',
      'UMMF::UML::MetaModel::OCL::Expressions::PropertyCallExp',
      'OCL::Expressions::PropertyCallExp',
      'UMMF::UML::MetaModel::OCL::Expressions::PropertyCallExp',
      'IntegerLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::IntegerLiteralExp',
      'OCL::Expressions::IntegerLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::IntegerLiteralExp',
      'AssociationEndCallExp',
      'UMMF::UML::MetaModel::OCL::Expressions::AssociationEndCallExp',
      'OCL::Expressions::AssociationEndCallExp',
      'UMMF::UML::MetaModel::OCL::Expressions::AssociationEndCallExp',
      'OclMessageArg',
      'UMMF::UML::MetaModel::OCL::Expressions::OclMessageArg',
      'OCL::Expressions::OclMessageArg',
      'UMMF::UML::MetaModel::OCL::Expressions::OclMessageArg',
      'ModelPropertyCallExp',
      'UMMF::UML::MetaModel::OCL::Expressions::ModelPropertyCallExp',
      'OCL::Expressions::ModelPropertyCallExp',
      'UMMF::UML::MetaModel::OCL::Expressions::ModelPropertyCallExp',
      'OclMessageExp',
      'UMMF::UML::MetaModel::OCL::Expressions::OclMessageExp',
      'OCL::Expressions::OclMessageExp',
      'UMMF::UML::MetaModel::OCL::Expressions::OclMessageExp',
      'OperationCallExp',
      'UMMF::UML::MetaModel::OCL::Expressions::OperationCallExp',
      'OCL::Expressions::OperationCallExp',
      'UMMF::UML::MetaModel::OCL::Expressions::OperationCallExp',
      'PrimitiveLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::PrimitiveLiteralExp',
      'OCL::Expressions::PrimitiveLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::PrimitiveLiteralExp',
      'AssociationClassEndCallExp',
      'UMMF::UML::MetaModel::OCL::Expressions::AssociationClassEndCallExp',
      'OCL::Expressions::AssociationClassEndCallExp',
      'UMMF::UML::MetaModel::OCL::Expressions::AssociationClassEndCallExp',
      'OclExpression',
      'UMMF::UML::MetaModel::OCL::Expressions::OclExpression',
      'OCL::Expressions::OclExpression',
      'UMMF::UML::MetaModel::OCL::Expressions::OclExpression',
      'VariableExp',
      'UMMF::UML::MetaModel::OCL::Expressions::VariableExp',
      'OCL::Expressions::VariableExp',
      'UMMF::UML::MetaModel::OCL::Expressions::VariableExp',
      'SendAction',
      'UMMF::UML::MetaModel::OCL::Expressions::SendAction',
      'OCL::Expressions::SendAction',
      'UMMF::UML::MetaModel::OCL::Expressions::SendAction',
      'BooleanLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::BooleanLiteralExp',
      'OCL::Expressions::BooleanLiteralExp',
      'UMMF::UML::MetaModel::OCL::Expressions::BooleanLiteralExp',
      'ElementImport',
      'UMMF::UML::MetaModel::Model_Management::ElementImport',
      'Model_Management::ElementImport',
      'UMMF::UML::MetaModel::Model_Management::ElementImport',
      'Subsystem',
      'UMMF::UML::MetaModel::Model_Management::Subsystem',
      'Model_Management::Subsystem',
      'UMMF::UML::MetaModel::Model_Management::Subsystem',
      'Package',
      'UMMF::UML::MetaModel::Model_Management::Package',
      'Model_Management::Package',
      'UMMF::UML::MetaModel::Model_Management::Package',
      'Model',
      'UMMF::UML::MetaModel::Model_Management::Model',
      'Model_Management::Model',
      'UMMF::UML::MetaModel::Model_Management::Model',
  ];

=head2 C<factory_map>

Returns an array ref that maps all UML Model names to Perl package names for this package.
Also includes short name to fully-qualified name mappings. 

=cut
sub factory_map { $factory_map; }
       
my $model;




=head2 C<model>

Returns the UML meta-model Model object for this Model.

=cut
sub model 
{
  unless ( $model ) {
    my $module = __PACKAGE__;
    $module =~ s@::@/@sg;
    my $file = $INC{$module . '.pm'};
    $file .= '.Model.Storable';
    
    print STDERR "deserializing UML Model: from $file\n";

    use UMMF::UML::Export::Storable;
    $model = Storable::retrieve($file);

    print STDERR "deserializing UML Model: DONE\n";


    print STDERR "reblessing UML Model\n";
    __rebless_tree($model, { }, { } );
    print STDERR "reblessing UML Model: DONE\n";
  }
  $model;
}


=head2 C<__rebless_tree>

Work around for errors like:

    Can't locate object method "parent" via package "UMMF::UML::MetaModel::Foundation::Core::Generalization" at /c/wct/ummf/1.0/bin/../lib/perl/UML/MetaMetaModel/Util.pm line 712.

=cut
# 'emacs
my %__rebless_tree_primitive =
map(($_ => 1),
    'HASH', 
    'ARRAY', 
    'SCALAR', 
    'CODE', 
    'GLOB',
    );

sub __rebless_tree
{
  no warnings;

  my ($x, $visited, $used) = @_;

  if ( my $ref = ref($x) ) {
    return if $visited->{$x} ++;

    # print STDERR "x = $x\n";

    # Rebless object.
    unless ( __rebless_tree_primitive{$ref} ) {
      # Does Storable::retrieve do this?
      unless ( $used->{$ref} ) {
	$used->{$ref} = 1;
	print STDERR "  use $ref\n";
	eval qq{ package main; use $ref; }; die $@ if $@;
      }

      # print STDERR "rebless $x, $ref\n";
      # REBLESS!!!
      bless($x, $ref);
    }

    # Traverse primitive
    if ( $ref eq 'Set::Object') {
      for my $y ( $x->members ) {
	__rebless_tree($y, $visited, $used);
      }
    }
    elsif ( $x =~ /HASH\(/ ) {
      for my $y ( values %$x ) {
	__rebless_tree($y, $visited, $used);
      }
    }
    elsif ( $x =~ /ARRAY\(/ ) {
      for my $y ( @$x ) {
	__rebless_tree($y, $visited, $used);
      }
    }
    elsif ( $x =~ /SCALAR\(/ ) {
      __rebless_tree($$x, $visited, $used);
    }
  }
}



my $factory;

=head2 C<factory>

Returns a factory for this Model.

=cut
sub factory 
{
  # $DB::single = 1;

  my $self = shift;
  unless ( $factory ) {
    use UMMF::UML::MetaMetaModel::Factory;
    $factory = UMMF::UML::MetaMetaModel::Factory->new('classMap' => $self->factory_map);
  }
  $factory;
}


############################################################################

1; # Is true!!!

############################################################################

### Keep these comments at end of file: kstephens@users.sourceforge.net 2003/04/06 ###
### Local Variables: ###
### mode:perl ###
### perl-indent-level:2 ###
### perl-continued-statement-offset:0 ###
### perl-brace-offset:0 ###
### perl-label-offset:0 ###
### End: ###

