# -*- perl -*-
# DO NOT EDIT - This file is generated by UMMF; http://ummf.sourceforge.net 
# From template: $Id: PerlTemplate.txt,v 1.74 2004/08/01 03:50:07 kstephens Exp $

package UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin;

#use 5.6.1;
use strict;
use warnings;

#################################################################
# Version
#

our $VERSION = do { my @r = (q{1.0} =~ /\d+/g); sprintf "%d." . "%03d" x $#r, @r };


#################################################################
# Documentation
#

=head1 NAME

UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin -- 

=head1 VERSION

1.0

=head1 SYNOPSIS

=head1 DESCRIPTION 

=head1 USAGE

=head1 EXPORT

=head1 METATYPE

L<UMMF::UML::MetaModel::Foundation::Core::Class|UMMF::UML::MetaModel::Foundation::Core::Class>

=head1 SUPERCLASSES

L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Pin|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Pin>




=head1 ATTRIBUTES

I<NO ATTRIBUTES>


=head1 ASSOCIATIONS


=head2 C<0..*> : C<outputPin> E<lt>---E<gt>  C<action> : UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action C<0..1>



=over 4

=item metatype = L<UMMF::UML::MetaModel::Foundation::Core::AssociationEnd|UMMF::UML::MetaModel::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action>

=item multiplicity = C<0..1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<1>

=item aggregation = C<composite>

=item visibility = C<public>

=item container_type = C<Set::Object>

=back


=head2 C<0..*> : C<bodyOutput> E<lt>----  C<> : UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::Clause C<0..*>



=over 4

=item metatype = L<UMMF::UML::MetaModel::Foundation::Core::AssociationEnd|UMMF::UML::MetaModel::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::Clause>

=item multiplicity = C<0..*>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<0>

=item aggregation = C<none>

=item visibility = C<private>

=item container_type = C<Set::Object>

=back


=head2 C<1> : C<source> E<lt>---E<gt>  C<flow> : UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow C<0..*>



=over 4

=item metatype = L<UMMF::UML::MetaModel::Foundation::Core::AssociationEnd|UMMF::UML::MetaModel::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow>

=item multiplicity = C<0..*>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<1>

=item aggregation = C<none>

=item visibility = C<public>

=item container_type = C<Set::Object>

=back


=head2 C<1..*> : C<leftSubinput> E<lt>----  C<> : UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Reduce::ReduceAction C<0..1>



=over 4

=item metatype = L<UMMF::UML::MetaModel::Foundation::Core::AssociationEnd|UMMF::UML::MetaModel::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Reduce::ReduceAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Reduce::ReduceAction>

=item multiplicity = C<0..1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<0>

=item aggregation = C<composite>

=item visibility = C<private>

=item container_type = C<Set::Object>

=back


=head2 C<0..*> : C<loopVariable> E<lt>---E<gt>  C<loop> : UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction C<0..1>



=over 4

=item metatype = L<UMMF::UML::MetaModel::Foundation::Core::AssociationEnd|UMMF::UML::MetaModel::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction>

=item multiplicity = C<0..1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<1>

=item aggregation = C<composite>

=item visibility = C<public>

=item container_type = C<Set::Object>

=back


=head2 C<0..*> : C<loopVariable> E<lt>----  C<> : UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Iterate::IterateAction C<0..1>



=over 4

=item metatype = L<UMMF::UML::MetaModel::Foundation::Core::AssociationEnd|UMMF::UML::MetaModel::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Iterate::IterateAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Iterate::IterateAction>

=item multiplicity = C<0..1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<0>

=item aggregation = C<composite>

=item visibility = C<private>

=item container_type = C<Set::Object>

=back


=head2 C<0..*> : C<argument> E<lt>---E<gt>  C<procedure> : UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure C<0..1>



=over 4

=item metatype = L<UMMF::UML::MetaModel::Foundation::Core::AssociationEnd|UMMF::UML::MetaModel::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure>

=item multiplicity = C<0..1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<1>

=item aggregation = C<composite>

=item visibility = C<public>

=item container_type = C<Set::Object>

=back


=head2 C<1..*> : C<rightSubinput> E<lt>----  C<> : UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Reduce::ReduceAction C<0..1>



=over 4

=item metatype = L<UMMF::UML::MetaModel::Foundation::Core::AssociationEnd|UMMF::UML::MetaModel::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Reduce::ReduceAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Reduce::ReduceAction>

=item multiplicity = C<0..1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<0>

=item aggregation = C<composite>

=item visibility = C<private>

=item container_type = C<Set::Object>

=back


=head2 C<1..*> : C<subinput> E<lt>----  C<> : UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Filter::FilterAction C<0..1>



=over 4

=item metatype = L<UMMF::UML::MetaModel::Foundation::Core::AssociationEnd|UMMF::UML::MetaModel::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Filter::FilterAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Filter::FilterAction>

=item multiplicity = C<0..1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<0>

=item aggregation = C<none>

=item visibility = C<private>

=item container_type = C<Set::Object>

=back


=head2 C<1..*> : C<subinput> E<lt>----  C<> : UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Iterate::IterateAction C<0..1>



=over 4

=item metatype = L<UMMF::UML::MetaModel::Foundation::Core::AssociationEnd|UMMF::UML::MetaModel::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Iterate::IterateAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Iterate::IterateAction>

=item multiplicity = C<0..1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<0>

=item aggregation = C<none>

=item visibility = C<private>

=item container_type = C<Set::Object>

=back


=head2 C<1..*> : C<subinput> E<lt>----  C<> : UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Map::MapAction C<0..1>



=over 4

=item metatype = L<UMMF::UML::MetaModel::Foundation::Core::AssociationEnd|UMMF::UML::MetaModel::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Map::MapAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Map::MapAction>

=item multiplicity = C<0..1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<0>

=item aggregation = C<composite>

=item visibility = C<private>

=item container_type = C<Set::Object>

=back


=head2 C<1..*> : C<suboutput> E<lt>----  C<> : UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Reduce::ReduceAction C<0..1>



=over 4

=item metatype = L<UMMF::UML::MetaModel::Foundation::Core::AssociationEnd|UMMF::UML::MetaModel::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Reduce::ReduceAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Reduce::ReduceAction>

=item multiplicity = C<0..1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<0>

=item aggregation = C<none>

=item visibility = C<private>

=item container_type = C<Set::Object>

=back


=head2 C<0..*> : C<suboutput> E<lt>----  C<> : UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Iterate::IterateAction C<0..1>



=over 4

=item metatype = L<UMMF::UML::MetaModel::Foundation::Core::AssociationEnd|UMMF::UML::MetaModel::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Iterate::IterateAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Iterate::IterateAction>

=item multiplicity = C<0..1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<0>

=item aggregation = C<none>

=item visibility = C<private>

=item container_type = C<Set::Object>

=back


=head2 C<0..*> : C<suboutput> E<lt>----  C<> : UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Map::MapAction C<0..1>



=over 4

=item metatype = L<UMMF::UML::MetaModel::Foundation::Core::AssociationEnd|UMMF::UML::MetaModel::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Map::MapAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Map::MapAction>

=item multiplicity = C<0..1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<0>

=item aggregation = C<none>

=item visibility = C<private>

=item container_type = C<Set::Object>

=back


=head2 C<1> : C<subtest> E<lt>----  C<> : UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Filter::FilterAction C<0..1>



=over 4

=item metatype = L<UMMF::UML::MetaModel::Foundation::Core::AssociationEnd|UMMF::UML::MetaModel::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Filter::FilterAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Collection_Actions::Filter::FilterAction>

=item multiplicity = C<0..1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<0>

=item aggregation = C<none>

=item visibility = C<private>

=item container_type = C<Set::Object>

=back


=head2 C<0..*> : C<testOutput> E<lt>----  C<> : UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::Clause C<0..*>



=over 4

=item metatype = L<UMMF::UML::MetaModel::Foundation::Core::AssociationEnd|UMMF::UML::MetaModel::Foundation::Core::AssociationEnd>

=item type = L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::Clause|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::Clause>

=item multiplicity = C<0..*>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<>

=item isNavigable = C<0>

=item aggregation = C<none>

=item visibility = C<private>

=item container_type = C<Set::Object>

=back



=head1 METHODS

=cut



#################################################################
# Dependencies
#





use Carp qw(croak confess);
use Set::Object 1.05;
use Class::Multimethods 1.70;
use Data::Dumper;
use Scalar::Util qw(weaken);
use UMMF::UML::MetaModel::__ObjectBase qw(:__ummf_array);


#################################################################
# Generalizations
#

use base qw(
  UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Pin



);


#################################################################
# Exports
#

our @EXPORT_OK = qw(
);
our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );





#################################################################
# Validation
#


=head2 C<__validate_type>

  UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin->__validate_type($value);

Returns true if C<$value> is a valid representation of L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin>.

=cut
sub __validate_type($$)
{
  my ($self, $x) = @_;

  no warnings;

  UNIVERSAL::isa($x, 'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin')  ;
}


=head2 C<__typecheck>

  UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin->__typecheck($value, $msg);

Calls C<confess()> with C<$msg> if C<<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin->__validate_type($value)>> is false.

=cut
sub __typecheck
{
  my ($self, $x, $msg) = @_;

  confess("typecheck: $msg: type '" . 'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin' . ": value '$x'")
    unless __validate_type($self, $x);
}


=head2 C<isaOutputPin>


Returns true if receiver is a L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin>.
Other receivers will return false.

=cut
sub isaOutputPin { 1 }


=head2 C<isaBehavioral_Elements__Actions__Action_Foundation__OutputPin>


Returns true if receiver is a L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin>.
Other receivers will return false.
This is the fully qualified version of the C<isaOutputPin> method.

=cut
sub isaBehavioral_Elements__Actions__Action_Foundation__OutputPin { 1 }


#################################################################
# Introspection
#

=head2 C<__model_name> 

  my $name = $obj_or_package->__model_name;

Returns the UML Model name (C<'Behavioral_Elements::Actions::Action_Foundation::OutputPin'>) for an object or package of
this Classifier.

=cut
sub __model_name { 'Behavioral_Elements::Actions::Action_Foundation::OutputPin' }



=head2 C<__isAbstract>

  $package->__isAbstract;

Returns C<0>.

=cut
sub __isAbstract { 0; }


my $__tangram_schema;
=head2 C<__tangram_schema>

  my $tangram_schema $obj_or_package->__tangram_schema

Returns a HASH ref that describes this Classifier for Tangram.

See L<UMMF::UML::Export::Perl::Tangram|UMMF::UML::Export::Perl::Tangram>

=cut
sub __tangram_schema
{
  my ($self) = @_;

  $__tangram_schema ||=
  {
   'classes' =>
   [
     'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin' =>
     {
       'table' => 'Behavioral_Elements__Actions__Action_Foundation__OutputPin',
       'abstract' => 0,
       'slots' => 
       { 
	 # Attributes
	 
	 # Associations
	 	 	       'action'
       => {
	 'type_impl' => 'ref',
         'class' => 'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action',

                  'null' => '1', 

                                    'col' => 'action', 

                                                                                                                   }
      ,
                  	 	                     	 	       'flow'
       => {
	 'type_impl' => 'iset',
         'class' => 'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow',

                           'table' => 'Behavioral_Elements__Actions__Action_Foundation__OutputPin__flow', 

                                                               'coll' => 'source',

                                                                               }
      ,
                  	 	                     	 	       'loop'
       => {
	 'type_impl' => 'ref',
         'class' => 'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction',

                  'null' => '1', 

                                    'col' => 'loop', 

                                                                                                                   }
      ,
                  	 	                     	 	       'procedure'
       => {
	 'type_impl' => 'ref',
         'class' => 'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure',

                  'null' => '1', 

                                    'col' => 'procedure', 

                                                                                                                   }
      ,
                  	 	                     	 	                     	 	                     	 	                     	 	                     	 	                     	 	                     	 	                     	 	                            },
       'bases' => [  'UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Pin',  ],
       'sql' => {

       },
     },
   ],

   'sql' =>
   {
    # Note Tangram::Ref::get_exporter() has
    # "UPDATE $table SET $self->{col} = $refid WHERE id = $id",
    # The id_col is hard-coded, 
    # Thus id_col will not work.
    #'id_col' => '__sid',
    #'class_col' => '__stype',
   },
     # 'set_id' => sub { }
     # 'get_id' => sub { }

      
  };
}


#################################################################
# Class Attributes
#


    

#################################################################
# Class Associations
#


    

#################################################################
# Initialization
#


=head2 C<___initialize>

Initialize all Attributes and AssociationEnds in a instance of this Classifier.
Does B<not> initalize slots in its Generalizations.

See also: C<__initialize>.

=cut
sub ___initialize
{
  my ($self) = @_;

  # Attributes



  # Associations

  # AssociationEnd 
  #  outputPin 0..*
  #  <--> 
  #  action 0..1 UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action.
    if ( defined $self->{'action'} ) {
    my $x = $self->{'action'};
    $self->{'action'} = undef;
    $self->set_action($x);
  }
  
  # AssociationEnd 
  #  source 1
  #  <--> 
  #  flow 0..* UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow.
    if ( defined $self->{'flow'} ) {
    my $x = $self->{'flow'};
        $self->{'flow'} = Set::Object->new();
        $self->set_flow(@$x);
  }
  
  # AssociationEnd 
  #  loopVariable 0..*
  #  <--> 
  #  loop 0..1 UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction.
    if ( defined $self->{'loop'} ) {
    my $x = $self->{'loop'};
    $self->{'loop'} = undef;
    $self->set_loop($x);
  }
  
  # AssociationEnd 
  #  argument 0..*
  #  <--> 
  #  procedure 0..1 UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure.
    if ( defined $self->{'procedure'} ) {
    my $x = $self->{'procedure'};
    $self->{'procedure'} = undef;
    $self->set_procedure($x);
  }
  

  $self;
}


my $__initialize_use;

=head2 C<__initialize>

Initialize all slots in this Classifier and all its Generalizations.

See also: C<___initialize>.

=cut
sub __initialize
{
  my ($self) = @_;

  # $DB::single = 1;

  unless ( ! $__initialize_use ) {
    $__initialize_use = 1;
    $self->__use('UMMF::UML::MetaModel::Foundation::Core::Element');
    $self->__use('UMMF::UML::MetaModel::Foundation::Core::ModelElement');
    $self->__use('UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Pin');
  }

  $self->UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin::___initialize;
  $self->UMMF::UML::MetaModel::Foundation::Core::Element::___initialize;
  $self->UMMF::UML::MetaModel::Foundation::Core::ModelElement::___initialize;
  $self->UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Pin::___initialize;

  $self;
}
      

=head2 C<__create>

Calls all <<create>> Methods for this Classifier and all Generalizations.

See also: C<___create>.

=cut
sub __create
{
  my ($self, @args) = @_;

  # $DB::single = 1;
  $self->UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin::___create(@args);
  $self->UMMF::UML::MetaModel::Foundation::Core::Element::___create();
  $self->UMMF::UML::MetaModel::Foundation::Core::ModelElement::___create();
  $self->UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Pin::___create();

  $self;
}




#################################################################
# Attributes
#




#################################################################
# Association
#


=for html <hr/>

=cut

#################################################################
# AssociationEnd outputPin <---> action
# type = UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action
# multiplicity = 0..1
# ordering = 

=head2 C<action>

  my $val = $obj->action;

Returns the AssociationEnd C<action> value of type L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action>.

=cut
sub action ($)
{
  my ($self) = @_;
		  
  $self->{'action'};
}


=head2 C<set_action>

  $obj->set_action($val);

Sets the AssociationEnd C<action> value.
C<$val> must of type L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action>.
Returns C<$obj>.

=cut
sub set_action ($$)
{
  my ($self, $val) = @_;
		  
  no warnings; # Use of uninitialized value in string ne at ...
		  
  my $old;
  if ( ($old = $self->{'action'}) ne $val ) { # Recursion lock

    if ( defined $val ) { $self->__use('UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action')->__typecheck($val, "UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin.action") }

    # Recursion lock
        $self->{'action'} = $val
    ;

    # Remove and add associations with other ends.
        
    $old->remove_outputPin($self) if $old;
    $val->add_outputPin($self)    if $val;

    }
		  
  $self;
}


=head2 C<add_action>

  $obj->add_action($val);

Adds the AssociationEnd C<action> value.
C<$val> must of type L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action>.
Throws exception if a value already exists.
Returns C<$obj>.

=cut
sub add_action ($$)
{
  my ($self, $val) = @_;

  no warnings; # Use of uninitialized value in string ne at ...

  my $old;
  if ( ($old = $self->{'action'}) ne $val ) { # Recursion lock
    $self->__use('UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action')->__typecheck($val, "UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin.action");
      
    # confess("UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin::action: too many")
    # if defined $self->{'action'};

    # Recursion lock
        $self->{'action'} = $val
    ;

    # Remove and add associations with other ends.
        
    $old->remove_outputPin($self) if $old;
    $val->add_outputPin($self)    if $val;

  
  }

  $self;
}


=head2 C<remove_action>

  $obj->remove_action($val);

Removes the AssociationEnd C<action> value C<$val>.
Returns C<$obj>.

=cut
sub remove_action ($$)
{
  my ($self, $val) = @_;

  no warnings; # Use of uninitialized value in string ne at ...

  my $old;
  if ( ($old = $self->{'action'}) eq $val ) { # Recursion lock
    $val = $self->{'action'} = undef;         # Recursion lock

    # Remove and add associations with other ends.
        
    $old->remove_outputPin($self) if $old;
    $val->add_outputPin($self)    if $val;

  
  }
}


=head2 C<clear_action>

  $obj->clear_action;

Clears the AssociationEnd C<action> links to L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action>.
Returns C<$obj>.

=cut
sub clear_action ($@)
{
  my ($self) = @_;

  my $old;
  if ( defined ($old = $self->{'action'}) ) { # Recursion lock
    my $val = $self->{'action'} = undef;      # Recursion lock

    # Remove and add associations with other ends.
        
    $old->remove_outputPin($self) if $old;
    $val->add_outputPin($self)    if $val;

    }

  $self;
}


=head2 C<count_action>

  $obj->count_action;

Returns the number of elements of type L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Action> associated with C<action>.

=cut
sub count_action ($)
{
  my ($self) = @_;

  my $x = $self->{'action'};

  defined $x ? 1 : 0;
}




=for html <hr/>

=cut

#################################################################
# AssociationEnd source <---> flow
# type = UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow
# multiplicity = 0..*
# ordering = 

=head2 C<flow>

  my @val = $obj->flow;
  my $ary_val = $obj->flow;

Returns the AssociationEnd C<flow> values of type L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow>.
In array context, returns all the objects in the Association.
In scalar context, returns an array ref of all the objects in the Association.

=cut
sub flow ($)
{
  my ($self) = @_;

    my $x = $self->{'flow'};

  # confess("Container for flow $x is not a blessed ref: " . Data::Dumper->new([ $self ], [qw($self)])->Maxdepth(2)->Dump()) if $x && ref($x) !~ /::/;
 
  wantarray ? ($x ? $x->members() : ()) : [ $x ? $x->members() : () ];
  
}


=head2 C<set_flow>

  $obj->set_flow(@val);

Sets the AssociationEnd C<flow> value.
Elements of C<@val> must of type L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow>.
Returns C<$obj>.

=cut
sub set_flow ($@)
{
  my ($self, @val) = @_;
  
  $self->clear_flow;
  $self->add_flow(@val);
}


=head2 C<add_flow>

  $obj->add_flow(@val);

Adds AssociationEnd C<flow> values.
Elements of C<@val> must of type L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow>.
Returns C<$obj>.

=cut
sub add_flow ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'flow'} ||= Set::Object->new();
    my $old; # Place holder for other MACRO.
  
  for my $val ( @val ) {
    # Recursion lock
        next if $x->includes($val);
        $self->__use('UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow')->__typecheck($val, "UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin.flow");

    # Recursion lock
        $x->insert($val);
    # weaken?
    
    # Remove and add associations with other ends.
        
    $old->remove_source($self) if $old;
    $val->add_source($self)    if $val;

    }
  
  $self;
}


=head2 C<remove_flow>

  $obj->remove_flow(@val);

Removes the AssociationEnd C<flow> values C<@val>.
Elements of C<@val> must of type L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow>.
Returns C<$obj>.

=cut
sub remove_flow ($@)
{
  my ($self, @val) = @_;
  
    my $x = $self->{'flow'} ||= Set::Object->new();
  
  for my $old ( @val ) {
    # Recursion lock
        next unless $x->includes($old);
    
    my $val = $old;
      
    $self->__use('UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow')->__typecheck($val, "UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin.flow");

    # Recursion lock
        $x->remove($old);
    
    $val = undef;

    # Remove associations with other ends.

        
    $old->remove_source($self) if $old;
    $val->add_source($self)    if $val;

  ;

  }
  
  $self;
}


=head2 C<clear_flow>

  $obj->clear_flow;

Clears the AssociationEnd C<flow> links to L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::DataFlow>.
Returns C<$obj>.

=cut
sub clear_flow ($) 
{
  my ($self) = @_;
  
    my $x = $self->{'flow'} ||= Set::Object->new();
  
  my $val; # Place holder for other MACRO.
  
    $self->{'flow'} = Set::Object->new(); # Recursion lock
  for my $old ( $x->members() ) {     # Recursion lock
  
    # Remove associations with other ends.

        
    $old->remove_source($self) if $old;
    $val->add_source($self)    if $val;

  ;

  }
  
  $self;
}


=head2 C<count_flow>

  $obj->count_flow;

Returns the number of elements associated with C<flow>.

=cut
sub count_flow ($)
{
  my ($self) = @_;

  my $x = $self->{'flow'};

    defined $x ? $x->size : 0;
  }




=for html <hr/>

=cut

#################################################################
# AssociationEnd loopVariable <---> loop
# type = UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction
# multiplicity = 0..1
# ordering = 

=head2 C<loop>

  my $val = $obj->loop;

Returns the AssociationEnd C<loop> value of type L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction>.

=cut
sub loop ($)
{
  my ($self) = @_;
		  
  $self->{'loop'};
}


=head2 C<set_loop>

  $obj->set_loop($val);

Sets the AssociationEnd C<loop> value.
C<$val> must of type L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction>.
Returns C<$obj>.

=cut
sub set_loop ($$)
{
  my ($self, $val) = @_;
		  
  no warnings; # Use of uninitialized value in string ne at ...
		  
  my $old;
  if ( ($old = $self->{'loop'}) ne $val ) { # Recursion lock

    if ( defined $val ) { $self->__use('UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction')->__typecheck($val, "UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin.loop") }

    # Recursion lock
        $self->{'loop'} = $val
    ;

    # Remove and add associations with other ends.
        
    $old->remove_loopVariable($self) if $old;
    $val->add_loopVariable($self)    if $val;

    }
		  
  $self;
}


=head2 C<add_loop>

  $obj->add_loop($val);

Adds the AssociationEnd C<loop> value.
C<$val> must of type L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction>.
Throws exception if a value already exists.
Returns C<$obj>.

=cut
sub add_loop ($$)
{
  my ($self, $val) = @_;

  no warnings; # Use of uninitialized value in string ne at ...

  my $old;
  if ( ($old = $self->{'loop'}) ne $val ) { # Recursion lock
    $self->__use('UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction')->__typecheck($val, "UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin.loop");
      
    # confess("UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin::loop: too many")
    # if defined $self->{'loop'};

    # Recursion lock
        $self->{'loop'} = $val
    ;

    # Remove and add associations with other ends.
        
    $old->remove_loopVariable($self) if $old;
    $val->add_loopVariable($self)    if $val;

  
  }

  $self;
}


=head2 C<remove_loop>

  $obj->remove_loop($val);

Removes the AssociationEnd C<loop> value C<$val>.
Returns C<$obj>.

=cut
sub remove_loop ($$)
{
  my ($self, $val) = @_;

  no warnings; # Use of uninitialized value in string ne at ...

  my $old;
  if ( ($old = $self->{'loop'}) eq $val ) { # Recursion lock
    $val = $self->{'loop'} = undef;         # Recursion lock

    # Remove and add associations with other ends.
        
    $old->remove_loopVariable($self) if $old;
    $val->add_loopVariable($self)    if $val;

  
  }
}


=head2 C<clear_loop>

  $obj->clear_loop;

Clears the AssociationEnd C<loop> links to L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction>.
Returns C<$obj>.

=cut
sub clear_loop ($@)
{
  my ($self) = @_;

  my $old;
  if ( defined ($old = $self->{'loop'}) ) { # Recursion lock
    my $val = $self->{'loop'} = undef;      # Recursion lock

    # Remove and add associations with other ends.
        
    $old->remove_loopVariable($self) if $old;
    $val->add_loopVariable($self)    if $val;

    }

  $self;
}


=head2 C<count_loop>

  $obj->count_loop;

Returns the number of elements of type L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Composite_Actions::LoopAction> associated with C<loop>.

=cut
sub count_loop ($)
{
  my ($self) = @_;

  my $x = $self->{'loop'};

  defined $x ? 1 : 0;
}




=for html <hr/>

=cut

#################################################################
# AssociationEnd argument <---> procedure
# type = UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure
# multiplicity = 0..1
# ordering = 

=head2 C<procedure>

  my $val = $obj->procedure;

Returns the AssociationEnd C<procedure> value of type L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure>.

=cut
sub procedure ($)
{
  my ($self) = @_;
		  
  $self->{'procedure'};
}


=head2 C<set_procedure>

  $obj->set_procedure($val);

Sets the AssociationEnd C<procedure> value.
C<$val> must of type L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure>.
Returns C<$obj>.

=cut
sub set_procedure ($$)
{
  my ($self, $val) = @_;
		  
  no warnings; # Use of uninitialized value in string ne at ...
		  
  my $old;
  if ( ($old = $self->{'procedure'}) ne $val ) { # Recursion lock

    if ( defined $val ) { $self->__use('UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure')->__typecheck($val, "UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin.procedure") }

    # Recursion lock
        $self->{'procedure'} = $val
    ;

    # Remove and add associations with other ends.
        
    $old->remove_argument($self) if $old;
    $val->add_argument($self)    if $val;

    }
		  
  $self;
}


=head2 C<add_procedure>

  $obj->add_procedure($val);

Adds the AssociationEnd C<procedure> value.
C<$val> must of type L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure>.
Throws exception if a value already exists.
Returns C<$obj>.

=cut
sub add_procedure ($$)
{
  my ($self, $val) = @_;

  no warnings; # Use of uninitialized value in string ne at ...

  my $old;
  if ( ($old = $self->{'procedure'}) ne $val ) { # Recursion lock
    $self->__use('UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure')->__typecheck($val, "UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin.procedure");
      
    # confess("UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::OutputPin::procedure: too many")
    # if defined $self->{'procedure'};

    # Recursion lock
        $self->{'procedure'} = $val
    ;

    # Remove and add associations with other ends.
        
    $old->remove_argument($self) if $old;
    $val->add_argument($self)    if $val;

  
  }

  $self;
}


=head2 C<remove_procedure>

  $obj->remove_procedure($val);

Removes the AssociationEnd C<procedure> value C<$val>.
Returns C<$obj>.

=cut
sub remove_procedure ($$)
{
  my ($self, $val) = @_;

  no warnings; # Use of uninitialized value in string ne at ...

  my $old;
  if ( ($old = $self->{'procedure'}) eq $val ) { # Recursion lock
    $val = $self->{'procedure'} = undef;         # Recursion lock

    # Remove and add associations with other ends.
        
    $old->remove_argument($self) if $old;
    $val->add_argument($self)    if $val;

  
  }
}


=head2 C<clear_procedure>

  $obj->clear_procedure;

Clears the AssociationEnd C<procedure> links to L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure>.
Returns C<$obj>.

=cut
sub clear_procedure ($@)
{
  my ($self) = @_;

  my $old;
  if ( defined ($old = $self->{'procedure'}) ) { # Recursion lock
    my $val = $self->{'procedure'} = undef;      # Recursion lock

    # Remove and add associations with other ends.
        
    $old->remove_argument($self) if $old;
    $val->add_argument($self)    if $val;

    }

  $self;
}


=head2 C<count_procedure>

  $obj->count_procedure;

Returns the number of elements of type L<UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure|UMMF::UML::MetaModel::Behavioral_Elements::Actions::Action_Foundation::Procedure> associated with C<procedure>.

=cut
sub count_procedure ($)
{
  my ($self) = @_;

  my $x = $self->{'procedure'};

  defined $x ? 1 : 0;
}







# End of Class OutputPin


=pod

=for html <hr/>

I<END OF DOCUMENT>

=cut

############################################################################

1; # is true!

############################################################################

### Keep these comments at end of file: kstephens@users.sourceforge.net 2003/04/06 ###
### Local Variables: ###
### mode:perl ###
### perl-indent-level:2 ###
### perl-continued-statement-offset:0 ###
### perl-brace-offset:0 ###
### perl-label-offset:0 ###
### End: ###

