# -*- perl -*-
# DO NOT EDIT - This file is generated by UMMF; http://ummf.sourceforge.net 
# From template: $Id: PerlTemplate.txt,v 1.74 2004/08/01 03:50:07 kstephens Exp $

package UMMF::UML::MetaModel::Model_Management::Subsystem;

#use 5.6.1;
use strict;
use warnings;

#################################################################
# Version
#

our $VERSION = do { my @r = (q{1.0} =~ /\d+/g); sprintf "%d." . "%03d" x $#r, @r };


#################################################################
# Documentation
#

=head1 NAME

UMMF::UML::MetaModel::Model_Management::Subsystem -- 

=head1 VERSION

1.0

=head1 SYNOPSIS

=head1 DESCRIPTION 

=head1 USAGE

=head1 EXPORT

=head1 METATYPE

L<UMMF::UML::MetaModel::Foundation::Core::Class|UMMF::UML::MetaModel::Foundation::Core::Class>

=head1 SUPERCLASSES

L<UMMF::UML::MetaModel::Model_Management::Package|UMMF::UML::MetaModel::Model_Management::Package>

L<UMMF::UML::MetaModel::Foundation::Core::Classifier|UMMF::UML::MetaModel::Foundation::Core::Classifier>




=head1 ATTRIBUTES


=head2 C<isInstantiable> : UMMF::UML::MetaModel::Foundation::Data_Types::Boolean 


=over 4

=item metatype = L<UMMF::UML::MetaModel::Foundation::Core::Attribute|UMMF::UML::MetaModel::Foundation::Core::Attribute>

=item type = L<UMMF::UML::MetaModel::Foundation::Data_Types::Boolean|UMMF::UML::MetaModel::Foundation::Data_Types::Boolean>

=item visibility = C<private>

=item multiplicity = C<1>

=item changeability = C<changeable>

=item targetScope = C<instance>

=item ordering = C<unordered>

=item initialValue = I<UNSPECIFIED>

=item container_type = C<Set::Object>

=back



=head1 ASSOCIATIONS

I<NO ASSOCIATIONS>


=head1 METHODS

=cut



#################################################################
# Dependencies
#





use Carp qw(croak confess);
use Set::Object 1.05;
use Class::Multimethods 1.70;
use Data::Dumper;
use Scalar::Util qw(weaken);
use UMMF::UML::MetaModel::__ObjectBase qw(:__ummf_array);


#################################################################
# Generalizations
#

use base qw(
  UMMF::UML::MetaModel::Model_Management::Package  UMMF::UML::MetaModel::Foundation::Core::Classifier



);


#################################################################
# Exports
#

our @EXPORT_OK = qw(
);
our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );





#################################################################
# Validation
#


=head2 C<__validate_type>

  UMMF::UML::MetaModel::Model_Management::Subsystem->__validate_type($value);

Returns true if C<$value> is a valid representation of L<UMMF::UML::MetaModel::Model_Management::Subsystem|UMMF::UML::MetaModel::Model_Management::Subsystem>.

=cut
sub __validate_type($$)
{
  my ($self, $x) = @_;

  no warnings;

  UNIVERSAL::isa($x, 'UMMF::UML::MetaModel::Model_Management::Subsystem')  ;
}


=head2 C<__typecheck>

  UMMF::UML::MetaModel::Model_Management::Subsystem->__typecheck($value, $msg);

Calls C<confess()> with C<$msg> if C<<UMMF::UML::MetaModel::Model_Management::Subsystem->__validate_type($value)>> is false.

=cut
sub __typecheck
{
  my ($self, $x, $msg) = @_;

  confess("typecheck: $msg: type '" . 'UMMF::UML::MetaModel::Model_Management::Subsystem' . ": value '$x'")
    unless __validate_type($self, $x);
}


=head2 C<isaSubsystem>


Returns true if receiver is a L<UMMF::UML::MetaModel::Model_Management::Subsystem|UMMF::UML::MetaModel::Model_Management::Subsystem>.
Other receivers will return false.

=cut
sub isaSubsystem { 1 }


=head2 C<isaModel_Management__Subsystem>


Returns true if receiver is a L<UMMF::UML::MetaModel::Model_Management::Subsystem|UMMF::UML::MetaModel::Model_Management::Subsystem>.
Other receivers will return false.
This is the fully qualified version of the C<isaSubsystem> method.

=cut
sub isaModel_Management__Subsystem { 1 }


#################################################################
# Introspection
#

=head2 C<__model_name> 

  my $name = $obj_or_package->__model_name;

Returns the UML Model name (C<'Model_Management::Subsystem'>) for an object or package of
this Classifier.

=cut
sub __model_name { 'Model_Management::Subsystem' }



=head2 C<__isAbstract>

  $package->__isAbstract;

Returns C<0>.

=cut
sub __isAbstract { 0; }


my $__tangram_schema;
=head2 C<__tangram_schema>

  my $tangram_schema $obj_or_package->__tangram_schema

Returns a HASH ref that describes this Classifier for Tangram.

See L<UMMF::UML::Export::Perl::Tangram|UMMF::UML::Export::Perl::Tangram>

=cut
sub __tangram_schema
{
  my ($self) = @_;

  $__tangram_schema ||=
  {
   'classes' =>
   [
     'UMMF::UML::MetaModel::Model_Management::Subsystem' =>
     {
       'table' => 'Model_Management__Subsystem',
       'abstract' => 0,
       'slots' => 
       { 
	 # Attributes
	 	       'isInstantiable'
       => {
	 'type_impl' => 'string',
                                             'col' => 'isInstantiable', 

                                                                                                                   }
      ,
         
	 # Associations
	        },
       'bases' => [  'UMMF::UML::MetaModel::Model_Management::Package',  'UMMF::UML::MetaModel::Foundation::Core::Classifier',  ],
       'sql' => {

       },
     },
   ],

   'sql' =>
   {
    # Note Tangram::Ref::get_exporter() has
    # "UPDATE $table SET $self->{col} = $refid WHERE id = $id",
    # The id_col is hard-coded, 
    # Thus id_col will not work.
    #'id_col' => '__sid',
    #'class_col' => '__stype',
   },
     # 'set_id' => sub { }
     # 'get_id' => sub { }

      
  };
}


#################################################################
# Class Attributes
#


      

#################################################################
# Class Associations
#


    

#################################################################
# Initialization
#


=head2 C<___initialize>

Initialize all Attributes and AssociationEnds in a instance of this Classifier.
Does B<not> initalize slots in its Generalizations.

See also: C<__initialize>.

=cut
sub ___initialize
{
  my ($self) = @_;

  # Attributes

    # Attribute isInstantiable
  if ( exists $self->{'isInstantiable'} ) {
    my $x = $self->{'isInstantiable'};
    $self->{'isInstantiable'} = undef;
    $self->set_isInstantiable($x);
  } else {
      }
  


  # Associations


  $self;
}


my $__initialize_use;

=head2 C<__initialize>

Initialize all slots in this Classifier and all its Generalizations.

See also: C<___initialize>.

=cut
sub __initialize
{
  my ($self) = @_;

  # $DB::single = 1;

  unless ( ! $__initialize_use ) {
    $__initialize_use = 1;
    $self->__use('UMMF::UML::MetaModel::Model_Management::Package');
    $self->__use('UMMF::UML::MetaModel::Foundation::Core::Namespace');
    $self->__use('UMMF::UML::MetaModel::Foundation::Core::Element');
    $self->__use('UMMF::UML::MetaModel::Foundation::Core::ModelElement');
    $self->__use('UMMF::UML::MetaModel::Foundation::Core::GeneralizableElement');
    $self->__use('UMMF::UML::MetaModel::Foundation::Core::Classifier');
  }

  $self->UMMF::UML::MetaModel::Model_Management::Subsystem::___initialize;
  $self->UMMF::UML::MetaModel::Model_Management::Package::___initialize;
  $self->UMMF::UML::MetaModel::Foundation::Core::Namespace::___initialize;
  $self->UMMF::UML::MetaModel::Foundation::Core::Element::___initialize;
  $self->UMMF::UML::MetaModel::Foundation::Core::ModelElement::___initialize;
  $self->UMMF::UML::MetaModel::Foundation::Core::GeneralizableElement::___initialize;
  $self->UMMF::UML::MetaModel::Foundation::Core::Classifier::___initialize;

  $self;
}
      

=head2 C<__create>

Calls all <<create>> Methods for this Classifier and all Generalizations.

See also: C<___create>.

=cut
sub __create
{
  my ($self, @args) = @_;

  # $DB::single = 1;
  $self->UMMF::UML::MetaModel::Model_Management::Subsystem::___create(@args);
  $self->UMMF::UML::MetaModel::Model_Management::Package::___create();
  $self->UMMF::UML::MetaModel::Foundation::Core::Namespace::___create();
  $self->UMMF::UML::MetaModel::Foundation::Core::Element::___create();
  $self->UMMF::UML::MetaModel::Foundation::Core::ModelElement::___create();
  $self->UMMF::UML::MetaModel::Foundation::Core::GeneralizableElement::___create();
  $self->UMMF::UML::MetaModel::Foundation::Core::Classifier::___create();

  $self;
}




#################################################################
# Attributes
#



=for html <hr/>

=cut

#################################################################
# Attribute isInstantiable
# type = UMMF::UML::MetaModel::Foundation::Data_Types::Boolean
# multiplicity = 1
# ordering = unordered
# ownerScope = instance
# initialValue = 

=head2 C<isInstantiable>

  my $val = $obj->isInstantiable;

Returns the L<UMMF::UML::MetaModel::Foundation::Data_Types::Boolean|UMMF::UML::MetaModel::Foundation::Data_Types::Boolean> value of Attribute C<isInstantiable>.

=cut
sub isInstantiable ($)
{
  my ($self) = @_;

  ;

  my $val = $self->{'isInstantiable'};

  ;

  $val;
}


=head2 C<set_isInstantiable>

  $obj->set_isInstantiable($val);

Sets the value of Attribute C<isInstantiable>.
C<$val> must be of type L<UMMF::UML::MetaModel::Foundation::Data_Types::Boolean|UMMF::UML::MetaModel::Foundation::Data_Types::Boolean> or C<undef>.
Returns C<$obj>.

=cut
sub set_isInstantiable ($$)
{
  my ($self, $val) = @_;

  ;

  if ( defined $val ) {
    ;
  }

    $self->{'isInstantiable'} = $val
  ;

  ;

  $self;
}


=head2 C<count_isInstantiable>

  $obj->count_isInstantiable;

Returns the number of elements (0 or 1) in C<isInstantiable>.

=cut
sub count_isInstantiable ($)
{
  my ($self) = @_;

  ;

  my $val = $self->{'isInstantiable'};

  ;

  defined $val ? 1 : 0;
}




#################################################################
# Association
#





# End of Class Subsystem


=pod

=for html <hr/>

I<END OF DOCUMENT>

=cut

############################################################################

1; # is true!

############################################################################

### Keep these comments at end of file: kstephens@users.sourceforge.net 2003/04/06 ###
### Local Variables: ###
### mode:perl ###
### perl-indent-level:2 ###
### perl-continued-statement-offset:0 ###
### perl-brace-offset:0 ###
### perl-label-offset:0 ###
### End: ###

